////////////////////////////////////////////////////////////////////////////////
// Copyright (c) Datacard Corporation.  All Rights Reserved.
//
// declarations for xps driver sdk c++ util items.
////////////////////////////////////////////////////////////////////////////////
#pragma once

#include <atlstr.h>
#include <msxml6.h>
#include <bidispl.h>
#include <vector>
#include <string>
#include "DXP01SDK.H"

namespace util
{
   struct PrinterStatusValues
   {
      CString  _clientID;
      int      _errorCode;
      int      _errorSeverity;
      CString  _errorString;
      CString  _printerData;
      int      _printerJobID;
      int      _windowsJobID;

      PrinterStatusValues() : _errorCode(0)
         , _errorSeverity(0)
         , _printerJobID(0)
         , _windowsJobID(0)
      {
      }
   };

   struct PrinterOptionsValues
   {
      CString  _colorPrintResolution;
      CString  _connectionPortType;
      CString  _connectionProtocol;
      CString  _embosserVersion;
      CString  _lockState;
      CString  _moduleEmbosser;
      CString  _monochromePrintResolution;
      CString  _optionDuplex;
      CString  _optionInputhopper;
      CString  _optionLocks;
      CString  _optionMagstripe;
      CString  _optionSmartcard;
      CString  _printerAddress;
      int      _printerMessageNumber;
      CString  _printerModel;
      CString  _printerSerialNumber;
      CString  _printerStatus;
      CString  _printerVersion;
      CString  _printHead;
      CString  _topcoatPrintResolution;

      PrinterOptionsValues() : _printerMessageNumber(0) {}
   };

   struct PrinterCardCountValues
   {
      int      _cardsPickedSinceCleaningCard;
      int      _cleaningCardsRun;
      int      _currentCompleted;
      int      _currentLost;
      int      _currentPicked;
      int      _currentPickedExceptionSlot;
      int      _currentPickedInputHopper1;
      int      _currentPickedInputHopper2;
      int      _currentPickedInputHopper3;
      int      _currentPickedInputHopper4;
      int      _currentPickedInputHopper5;
      int      _currentPickedInputHopper6;
      int      _currentRejected;
      CString  _printerStatus;
      int      _totalCompleted;
      int      _totalLost;
      int      _totalPicked;
      int      _totalPickedExceptionSlot;
      int      _totalPickedInputHopper1;
      int      _totalPickedInputHopper2;
      int      _totalPickedInputHopper3;
      int      _totalPickedInputHopper4;
      int      _totalPickedInputHopper5;
      int      _totalPickedInputHopper6;
      int      _totalRejected;

      PrinterCardCountValues() : _cardsPickedSinceCleaningCard(0)
         , _cleaningCardsRun(0)
         , _currentCompleted(0)
         , _currentLost(0)
         , _currentPicked(0)
         , _currentPickedExceptionSlot(0)
         , _currentPickedInputHopper1(0)
         , _currentPickedInputHopper2(0)
         , _currentPickedInputHopper3(0)
         , _currentPickedInputHopper4(0)
         , _currentPickedInputHopper5(0)
         , _currentPickedInputHopper6(0)
         , _currentRejected(0)
         , _totalCompleted(0)
         , _totalLost(0)
         , _totalPicked(0)
         , _totalPickedExceptionSlot(0)
         , _totalPickedInputHopper1(0)
         , _totalPickedInputHopper2(0)
         , _totalPickedInputHopper3(0)
         , _totalPickedInputHopper4(0)
         , _totalPickedInputHopper5(0)
         , _totalPickedInputHopper6(0)
         , _totalRejected(0)
      {}
   };

   struct SuppliesValues
   {
      CString _indentRibbon;
      short   _indentRibbonRemaining;
      CString _printerStatus;
      CString _printRibbonType;
      CString _printRibbonSerialNumber;
      long    _printRibbonPartNumber;
      CString _printRibbonLotCode;
      short   _ribbonRemaining;
      short   _topperRibbonRemaining;
      CString _topperRibbonType;

      SuppliesValues() : _ribbonRemaining(0)
         , _indentRibbonRemaining(0)
         , _topperRibbonRemaining(0)
      {}
   };

   struct JobStatusValues
   {
      CString  _clientID;
      int      _windowsJobID;
      int      _printerJobID;
      CString  _jobState;
      int      _jobRestartCount;

      JobStatusValues() : _windowsJobID(0)
         , _printerJobID(0)
         , _jobRestartCount(0)
      {
      }
   };

   struct SmartcardResponseValues
   {
      std::wstring               _protocol;
      std::vector <std::wstring> _states;
      std::wstring               _status;
      std::wstring               _base64string;
      std::vector <byte>         _bytesFromBase64String;
   };

   CString Win32ErrorString(const long errorCode);
   void EchoCurrentTime(CString msg);

   CString FormatPrinterActionXML(
      const int actionID,
      const int printerJobID,
      const int errorCode);

   CString GetPrinterStatusXML(CComPtr <IBidiRequest>& bidiRequest);
   CString GetPrinterStatusXML(CComPtr <IBidiSpl>& bidiSpl);
   PrinterStatusValues ParsePrinterStatusXML(const CString printerStatusXML);

   CString GetPrinterOptionsXML(CComPtr <IBidiSpl>& bidiSpl);
   PrinterOptionsValues ParsePrinterOptionsXML(const CString printerOptionsXML);

   CString GetPrinterCardCountsXML(CComPtr <IBidiSpl>& bidiSpl);
   PrinterCardCountValues ParsePrinterCardCountsXML(const CString printerCardCountsXML);

   CString GetSDKVersionXML(CComPtr <IBidiSpl>& bidiSpl);
   CString ParseSDKVersionXML(const CString printerSDKVersionXML);

   CString GetPrinterActionReturnXML(
      CComPtr <IBidiSpl>& bidiSpl, 
      CString actionString);

   CString GetJobStatusXML(CComPtr <IBidiSpl>& bidiSpl, const DWORD printerJobID);
   JobStatusValues ParseJobStatusXML(const CString jobStatusXML);

   CString GetSuppliesXML(CComPtr<IBidiSpl>& bidiSpl);
   SuppliesValues ParseSuppliesXML(const CString suppliesXML);

   SmartcardResponseValues ParseSmartcardResponseXML(const CString smartcardReponseXML);

   void ParseMagstripeStrings(
      const CString  printerStatusXML,
      CString&       track1,
      CString&       track2,
      CString&       track3,
      bool           bJISRequest);

   void WaitUntilJobSpooled(const HANDLE printerHandle, const DWORD windowsJobID);

   void PrintTextAndGraphics(
      CString        printerName,
      const HANDLE   printerHandle,
      DWORD&         windowsJobID,
      const bool     duplex);

   void PrintText(CString printerName, const bool duplex);

   int     IntFromXml(CComPtr <IXMLDOMDocument2>& doc, const PWCHAR xpathExpression);
   CString StringFromXml(CComPtr <IXMLDOMDocument2>& doc, const PWCHAR xpathExpression);
   bool    BoolFromXml(CComPtr <IXMLDOMDocument2>& doc, const PWCHAR xpathExpression);
   ULONG   UlongFromXml(CComPtr <IXMLDOMDocument2>& doc, const PWCHAR xpathExpression);
}
