﻿////////////////////////////////////////////////////////////////////////////////
// Copyright (c) Datacard Corporation.  All Rights Reserved.
//
// xps card printer SDK: csharp 'control' sample
////////////////////////////////////////////////////////////////////////////////
using System;
using System.Threading;
using dxp01sdk;

public enum Commands
{
    undefined,
    cancellalljobs
}

public class CommandLineOptions
{
    public CommandLineOptions() { command = Commands.undefined; }
    public string printerName;
    public Commands command;
}

class control
{
    private static void Usage()
    {
        Console.WriteLine();
        Console.WriteLine("printer_control.exe controls the printer through the printer driver.");
        Console.WriteLine();
        Console.WriteLine("printer_control.exe -n <printername> <command>");
        Console.WriteLine("  -n <printername>. Required. Try -n \"XPS Card Printer\".");
        Console.WriteLine("  -c cancel all jobs: Send a command to the printer to clear any print job");
        Console.WriteLine("                    from the printer. This does not delete spooled Windows");
        Console.WriteLine("                    print jobs.");
        Console.WriteLine();
        Environment.Exit(-1);
    }

    static CommandLineOptions GetCommandlineOptions(string[] args)
    {
        CommandLineOptions commandLineOptions = new CommandLineOptions();
        CommandLine.Utility.Arguments arguments = new CommandLine.Utility.Arguments(args);

        if (string.IsNullOrEmpty(arguments["n"]))
        {
            Usage();
        }

        // check for -n with no printer name:
        bool boolVal = false;
        if (Boolean.TryParse(arguments["n"], out boolVal))
        {
            Usage();
        }
        commandLineOptions.printerName = arguments["n"];

        if (!string.IsNullOrEmpty(arguments["c"]))
        {
            commandLineOptions.command = Commands.cancellalljobs;
        }

        if (Commands.undefined == commandLineOptions.command)
        {
            Usage();
        }

        return commandLineOptions;
    }

    ////////////////////////////////////////////////////////////////////////////
    // CancelAllJobs()
    //
    // using the IBidiSpl interface, issue a 'cancel all jobs' command. This is
    // simply a dxp01sdk::Cancel with both printerJobID and errorCode set to zero.
    //
    ////////////////////////////////////////////////////////////////////////////
    static void CancelAllJobs(BidiSplWrap bidiSpl)
    {
        string xmlFormat = strings.PRINTER_ACTION_XML;
        string input = string.Format(xmlFormat, (int)Actions.Cancel, 0, 0);
        bidiSpl.SetPrinterData(strings.PRINTER_ACTION, input);
    }

    ////////////////////////////////////////////////////////////////////////////
    // Main()
    ////////////////////////////////////////////////////////////////////////////
    static void Main(string[] args)
    {
        CommandLineOptions commandLineOptions = GetCommandlineOptions(args);

        BidiSplWrap bidiSpl = new BidiSplWrap();
        bidiSpl.BindDevice(commandLineOptions.printerName);

        if (Commands.cancellalljobs == commandLineOptions.command)
        {
            CancelAllJobs(bidiSpl);
            Console.WriteLine("all jobs canceled for '{0}'", commandLineOptions.printerName);
        }

        bidiSpl.UnbindDevice();
    }
}
