﻿////////////////////////////////////////////////////////////////////////////////
// Copyright (c) Datacard Corporation.  All Rights Reserved.
//
// xps card printer SDK: csharp status sample
////////////////////////////////////////////////////////////////////////////////
using System;
using System.Threading;
using dxp01sdk;

public class CommandLineOptions
{
    public string printerName;
    public bool jobStatus;
}

class status
{
    private static void Usage()
    {
        Console.WriteLine();
        Console.WriteLine("status.exe demonstrates using interactive mode to retrieve printer options,");
        Console.WriteLine("  printer messages, and job status.");
        Console.WriteLine();
        Console.WriteLine("status.exe -n <printername>");
        Console.WriteLine("  -n <printername>. Required. Try -n \"XPS Card Printer\".");
        Console.WriteLine("  -j Optional. Submit a print job and display job status.");
        Console.WriteLine();
        Environment.Exit(-1);
    }

    static CommandLineOptions GetCommandlineOptions(string[] args)
    {
        if (args.Length == 0) Usage();

        CommandLineOptions commandLineOptions = new CommandLineOptions();
        CommandLine.Utility.Arguments arguments = new CommandLine.Utility.Arguments(args);

        if (string.IsNullOrEmpty(arguments["n"])) { Usage(); }

        // we might have a -n with no printer name:
        bool boolVal = false;
        if (Boolean.TryParse(arguments["n"], out boolVal)) { Usage(); }

        if (!string.IsNullOrEmpty(arguments["j"])) { commandLineOptions.jobStatus = true; }

        commandLineOptions.printerName = arguments["n"];

        return commandLineOptions;
    }

    static void CancelJob(BidiSplWrap bidiSpl, int printerJobID, int errorCode)
    {
        string xmlFormat = strings.PRINTER_ACTION_XML;
        string input = string.Format(xmlFormat, (int)Actions.Cancel, printerJobID, errorCode);
        bidiSpl.SetPrinterData(strings.PRINTER_ACTION, input);
    }

    static void PollForJobCompletion(BidiSplWrap bidiSpl, int printerJobID)
    {
        Console.WriteLine("JobStatus:");
        for (; true; )
        {
            string temp = strings.JOB_STATUS_XML;
            string jobInfoXML = string.Format(temp, printerJobID);
            string jobStatusXML = bidiSpl.GetPrinterData(strings.JOB_STATUS, jobInfoXML);
            JobStatusValues jobStatusValues = Util.ParseJobStatusXML(jobStatusXML);
            Console.WriteLine("  ClientID:          " + jobStatusValues._clientID);
            Console.WriteLine("  WindowsJobID:      " + jobStatusValues._windowsJobID);
            Console.WriteLine("  PrinterJobID:      " + jobStatusValues._printerJobID);
            Console.WriteLine("  JobState:          " + jobStatusValues._jobState);
            Console.WriteLine("  JobRestartCount:   " + jobStatusValues._jobRestartCount);
            Console.WriteLine();
            if (strings.JOB_ACTIVE != jobStatusValues._jobState)
            {
                Console.WriteLine();
                return;
            }
            Thread.Sleep(1000);
        }
    }

    private static void StartJob(BidiSplWrap bidiSpl, ref int printerJobID)
    {
        string printerStatusXML = bidiSpl.SetPrinterData(strings.STARTJOB);
        PrinterStatusValues printerStatusValues = Util.ParsePrinterStatusXML(printerStatusXML);
        if (0 != printerStatusValues._errorCode)
        {
            Console.WriteLine("StartJob() fail; canceling the job. Printer Error: " + printerStatusValues._errorString);
            CancelJob(bidiSpl, printerStatusValues._printerJobID, printerStatusValues._errorCode);
            bidiSpl.UnbindDevice();
            Environment.Exit(-1);
        }
        else
        {
            printerJobID = printerStatusValues._printerJobID;
        }
    }

    private static void DisplayPrinterOptionsValues(PrinterOptionsValues vals)
    {
        Console.WriteLine("Options:");
        Console.WriteLine("  ColorPrintResolution:      " + vals._colorPrintResolution);
        Console.WriteLine("  ConnectionPortType:        " + vals._connectionPortType);
        Console.WriteLine("  ConnectionProtocol:        " + vals._connectionProtocol);
        Console.WriteLine("  EmbosserVersion:           " + vals._embosserVersion);
        Console.WriteLine("  LockState:                 " + vals._lockState);
        Console.WriteLine("  ModuleEmbosser:            " + vals._moduleEmbosser);
        Console.WriteLine("  MonochromePrintResolution: " + vals._monochromePrintResolution);
        Console.WriteLine("  OptionDuplex:              " + vals._optionDuplex);
        Console.WriteLine("  OptionInputhopper:         " + vals._optionInputhopper);
        Console.WriteLine("  OptionLocks:               " + vals._optionLocks);
        Console.WriteLine("  OptionMagstripe:           " + vals._optionMagstripe);
        Console.WriteLine("  OptionSmartcard:           " + vals._optionSmartcard);
        Console.WriteLine("  PrinterAddress:            " + vals._printerAddress);
        Console.WriteLine("  PrinterMessageNumber:      " + vals._printerMessageNumber);
        Console.WriteLine("  PrinterModel:              " + vals._printerModel);
        Console.WriteLine("  PrinterSerialNumber:       " + vals._printerSerialNumber);
        Console.WriteLine("  PrinterStatus:             " + vals._printerStatus);
        Console.WriteLine("  PrinterVersion:            " + vals._printerVersion);
        Console.WriteLine("  PrintHead:                 " + vals._printHead);
        Console.WriteLine("  TopcoatPrintResolution:    " + vals._topcoatPrintResolution);
        Console.WriteLine();
    }

    private static void DisplayPrinterStatusValues(PrinterStatusValues vals)
    {
        Console.WriteLine("Status:");
        Console.WriteLine("  ClientID:      " + vals._clientID);
        Console.WriteLine("  ErrorCode:     " + vals._errorCode);
        Console.WriteLine("  ErrorSeverity: " + vals._errorSeverity);
        Console.WriteLine("  ErrorString:   " + vals._errorString);
        Console.WriteLine("  PrinterData:   " + vals._dataFromPrinter);
        Console.WriteLine("  PrinterJobID:  " + vals._printerJobID);
        Console.WriteLine("  WindowsJobID:  " + vals._windowsJobID);
        Console.WriteLine();
    }

    private static void DisplayPrinterCounterValues(PrinterCounterStatus vals)
    {
        Console.WriteLine("Counts:");
        Console.WriteLine("  CardsPickedSinceCleaningCard:  " + vals._cardsPickedSinceCleaningCard);
        Console.WriteLine("  CleaningCardsRun:              " + vals._cleaningCardsRun);
        Console.WriteLine("  CurrentCompleted:              " + vals._currentCompleted);
        Console.WriteLine("  CurrentLost:                   " + vals._currentLost);
        Console.WriteLine("  CurrentPicked:                 " + vals._currentPicked);
        Console.WriteLine("  CurrentPickedExceptionSlot:    " + vals._currentPickedExceptionSlot);
        Console.WriteLine("  CurrentPickedInputHopper1:     " + vals._currentPickedInputHopper1);
        Console.WriteLine("  CurrentPickedInputHopper2:     " + vals._currentPickedInputHopper2);
        Console.WriteLine("  CurrentPickedInputHopper3:     " + vals._currentPickedInputHopper3);
        Console.WriteLine("  CurrentPickedInputHopper4:     " + vals._currentPickedInputHopper4);
        Console.WriteLine("  CurrentPickedInputHopper5:     " + vals._currentPickedInputHopper5);
        Console.WriteLine("  CurrentPickedInputHopper6:     " + vals._currentPickedInputHopper6);
        Console.WriteLine("  CurrentRejected:               " + vals._currentRejected);
        Console.WriteLine("  PrinterStatus:                 " + vals._printerStatus);
        Console.WriteLine("  TotalCompleted:                " + vals._totalCompleted);
        Console.WriteLine("  TotalLost:                     " + vals._totalLost);
        Console.WriteLine("  TotalPicked:                   " + vals._totalPicked);
        Console.WriteLine("  TotalPickedExceptionSlot:      " + vals._totalPickedExceptionSlot);
        Console.WriteLine("  TotalPickedInputHopper1:       " + vals._totalPickedInputHopper1);
        Console.WriteLine("  TotalPickedInputHopper2:       " + vals._totalPickedInputHopper2);
        Console.WriteLine("  TotalPickedInputHopper3:       " + vals._totalPickedInputHopper3);
        Console.WriteLine("  TotalPickedInputHopper4:       " + vals._totalPickedInputHopper4);
        Console.WriteLine("  TotalPickedInputHopper5:       " + vals._totalPickedInputHopper5);
        Console.WriteLine("  TotalPickedInputHopper6:       " + vals._totalPickedInputHopper6);
        Console.WriteLine("  TotalRejected:                 " + vals._totalRejected);
        Console.WriteLine();
    }

    private static void DisplaySuppliesValues(SuppliesValues vals)
    {
        Console.WriteLine("Supplies:");

        Console.WriteLine("  IndentRibbon:              " + vals._indentRibbon);
        Console.WriteLine("  IndentRibbonRemaining:     " + vals._indentRibbonRemaining + "%");
        Console.WriteLine("  PrinterStatus:             " + vals._printerStatus);
        Console.WriteLine("  PrintRibbonLotCode:        " + vals._printRibbonLotCode);
        Console.WriteLine("  PrintRibbonPartNumber:     " + vals._printRibbonPartNumber);
        Console.WriteLine("  PrintRibbonSerialNumber:   " + vals._printRibbonSerialNumber);
        Console.WriteLine("  PrintRibbonType:           " + vals._printRibbonType);
        Console.WriteLine("  RibbonRemaining:           " + vals._ribbonRemaining + "%");
        Console.WriteLine("  TopperRibbonRemaining:     " + vals._topperRibbonRemaining + "%");
        Console.WriteLine("  TopperRibbonType:          " + vals._topperRibbonType);
        
        Console.WriteLine();
    }

    private static void CardJobWithStatus(
        BidiSplWrap bidiSpl,
        CommandLineOptions commandLineOptions,
        PrinterOptionsValues printerOptionsValues)
    {
        if ("Ready" != printerOptionsValues._printerStatus)
        {
            Console.WriteLine(commandLineOptions.printerName + " is not ready.");
            bidiSpl.UnbindDevice();
            Environment.Exit(-1);
        }

        int printerJobID = 0;
        StartJob(bidiSpl, ref printerJobID);

        string printerStatusXML = bidiSpl.GetPrinterData(strings.PRINTER_MESSAGES);
        PrinterStatusValues printerStatusValues = Util.ParsePrinterStatusXML(printerStatusXML);
        DisplayPrinterStatusValues(printerStatusValues);

        bidiSpl.SetPrinterData(strings.ENDJOB);

        PollForJobCompletion(bidiSpl, printerJobID);
    }

    ////////////////////////////////////////////////////////////////////////////////
    // Main()
    ////////////////////////////////////////////////////////////////////////////////
    static void Main(string[] args)
    {
        CommandLineOptions commandLineOptions = GetCommandlineOptions(args);

        BidiSplWrap bidiSpl = new BidiSplWrap();
        bidiSpl.BindDevice(commandLineOptions.printerName);

        string sdkVersion = bidiSpl.GetPrinterData(strings.SDK_VERSION);
        Console.WriteLine(Environment.NewLine + "SDK Version: " + Util.ParseSDKVersionXML(sdkVersion) + Environment.NewLine);

        string printerStatusXML = bidiSpl.GetPrinterData(strings.PRINTER_MESSAGES);
        PrinterStatusValues printerStatusValues = Util.ParsePrinterStatusXML(printerStatusXML);
        DisplayPrinterStatusValues(printerStatusValues);

        string printerOptionsXML = bidiSpl.GetPrinterData(strings.PRINTER_OPTIONS2);
        PrinterOptionsValues printerOptionsValues = Util.ParsePrinterOptionsXML(printerOptionsXML);
        DisplayPrinterOptionsValues(printerOptionsValues);

        string printerCardCountXML = bidiSpl.GetPrinterData(strings.COUNTER_STATUS2);
        PrinterCounterStatus printerCounterStatusValues = Util.ParsePrinterCounterStatusXML(printerCardCountXML);
        DisplayPrinterCounterValues(printerCounterStatusValues);

        string suppliesXML = bidiSpl.GetPrinterData(strings.SUPPLIES_STATUS2);
        SuppliesValues suppliesValues = Util.ParseSuppliesXML(suppliesXML);
        DisplaySuppliesValues(suppliesValues);

        if (commandLineOptions.jobStatus)
        {
            CardJobWithStatus(
                bidiSpl,
                commandLineOptions,
                printerOptionsValues);
        }

        bidiSpl.UnbindDevice();
    }
}
