////////////////////////////////////////////////////////////////////////////////
// Copyright (c) Datacard Corporation.  All Rights Reserved.
//
////////////////////////////////////////////////////////////////////////////////
package com.sun.jna.examples;

import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URL;

import com.datacard.xpsprinter.sdk.examples.JavaPrint;
import com.datacard.xpsprinter.sdk.examples.Magstripe;
import com.datacard.xpsprinter.sdk.examples.PrinterCounterStatus;
import com.datacard.xpsprinter.sdk.examples.PrinterOptions;
import com.datacard.xpsprinter.sdk.examples.PrinterSuppliesStatus;
import com.datacard.xpsprinter.sdk.examples.SmartCard;
import com.datacard.xpsprinter.sdk.examples.Util;
import com.sun.jna.Library;
import com.sun.jna.Native;

public class XPS_Java_SDK
{
   // buffer size for all the interop communications:
   public final static int    BUFFSIZE   = 10240;

   // Java interop DLL:
   public final static String INTEROPDLL = "dxp01sdk_IBidiSpl_interop";

   // xps printer interface and functions
   public interface XpsDriverInteropLib extends Library
   {

      XpsDriverInteropLib INSTANCE = (XpsDriverInteropLib) Native.loadLibrary(
                                         INTEROPDLL, XpsDriverInteropLib.class);

      int GetSDKVersion(String printerName, byte returnBuffer[],
            int bufferSize[]);

      // New
      int GetPrinterOptions2(String printerName, byte returnBuffer[],
            int bufferSize[]);

      // New
      int GetPrinterCounterStatus2(String printerName, byte returnBuffer[],
            int bufferSize[]);

      // New
      int GetPrinterSuppliesStatus(String printerName, byte returnBuffer[],
            int bufferSize[]);

      int StartJob(String printerName, byte returnXML[], int sizeOfReturnXML[]);

      int MagstripeRead(String printerName, byte dataBuffer[], int bufferSize[]);

      int MagstripeEncode(String printerName, String track1Data,
            int sizeOfTrack1Data, String track2Data, int sizeOfTrack2Data,
            String track3Data, int sizeOfTrack3Data, byte returnXML[],
            int sizeOfReturnXML[]);

      boolean SendResponseToPrinter(String aPrinterName, int command,
            int printerJobID, int errorCode);

      int SmartCardPark(String aPrinterName, byte returnXML[],
            int sizeOfReturnXML[]);

      int SCardConnect(String aPrinterName, int connectType, int protocal[]);

      int SCardDisConnect(String aPrinterName, int disposition);

      int SCardStatus(String aPrinterName, int states[],
            int numberStatesReturned[], int protocal[], byte returnATRBytes[],
            int numATRBytesReturned[]);

      int SCardGetAttrib(String aPrinterName, int attrID,
            byte attrBytesBuffer[], int attrBytesBufferSize[]);

      int SCardTransmit(String aPrinterName, byte sendBytesBuffer[],
            int sendBytesBufferSize, byte receivedBytesBuffer[],
            int receivedBytesBufferSize[]);

      int ResumeJob(String printerName, int printerJobID, int errorCode,
            byte returnXML[], int sizeOfReturnXML[]);

      int CancelJob(String printerName, int printerJobID, int errorCode,
            byte returnXML[], int sizeOfReturnXML[]);

      int EndJob(String aPrinterName);

      int GetJobStatusXML(String aPrinterName, int aPrinterJobID,
            byte returnXML[], int sizeOfReturnXML[]);

      // New

      int GetPrinterStatus(String schemaString, String printerName,
            byte returnBuffer[], int bufferSize[]);
      // ----------------------------

   }

   // make sure JNA can find xps printer interface dxp01sdk_IBidiSpl_interop.dll
   static
   {
      System.setProperty("jna.library.path",
            ".;" + System.getProperty("os.arch"));
      extractDll(System.getProperty("os.arch"), INTEROPDLL);
   }

   public static void main(String[] args)
   {
      String printerName = "";
      int i = 0;
      String arg;

      // parse the command line arguments
      while (i < args.length && args[i].startsWith("-"))
      {
         arg = args[i++];

         // use this type of check for arguments that require arguments
         if (arg.equalsIgnoreCase("-n"))
         {
            if (i < args.length)
            {
               printerName = args[i++];
            }
            else
            {
               System.err.println("-n requires a printername");
               break;
            }
         } // use this type of check for "wordy" arguments
         else if (arg.startsWith("-mr"))
         {
            System.out.format(
                  "Magstripe Read Demo\n\noptions ---------------------\nprint: %b\nisLandscape: %b\nisDuplex: %b\n",
                  (arg.length() > 3 && arg.contains("p")), arg.contains("l"),
                  arg.contains("d"));
            Magstripe magStripe = new Magstripe(printerName);
            magStripe.Read((arg.length() > 3 && arg.contains("p")),
                  arg.contains("l"), arg.contains("d"));
         }
         else if (arg.startsWith("-me"))
         {
            System.out.format(
                  "Magstripe Encode Demo\n\noptions ---------------------\nmagstripe read: %b\nprint: %b\nisLandscape: %b\nisDuplex: %b\nJIS magnetic: %b\n",
                  (arg.length() > 3 && arg.contains("mr")),
                  (arg.length() > 3 && arg.contains("p")), arg.contains("l"),
                  arg.contains("d"), arg.contains("j"));
            Magstripe magStripe = new Magstripe(printerName);
            boolean isJIS = arg.contains("j");
            if (!isJIS)
            {
               magStripe.Encode(
                     " !#$%&\'\"()*+,-./0123456789:;<=>@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_ !#$%&\'()*+,",
                     "0123456789:;<=>0123456789:;<=>0123456",
                     "0123456789:;<=>0123456789:;<=>0123456789:;<=>0123456789:;<=>0123456789:;<=>0123456789:;<=>0123456789:;<=",
                     (arg.length() > 3 && arg.contains("mr")),
                     (arg.length() > 3 && arg.contains("p")),
                     arg.contains("l"), arg.contains("d"));
            }
            else
            {
               // You need make sure your printer has JIS magstripe head and any
               // necessary
               // setting. The key here is the data should put into track 3
               // only.
               String nttData = "!\"#$%&\'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNO";
               magStripe.Encode("", "", nttData,
                     (arg.length() > 3 && arg.contains("mr")),
                     (arg.length() > 3 && arg.contains("p")),
                     arg.contains("l"), arg.contains("d"));
            }

         }
         else if (arg.equalsIgnoreCase("-s"))
         {
            System.out.println("Printer Options Demo");
            PrinterOptions printerOptions = new PrinterOptions(printerName);
            printerOptions.GetPrinterOptions();

         }
         else if (arg.equalsIgnoreCase("-cc"))
         {
            System.out.println("Printer Card Counts Demo");
            PrinterCounterStatus printerOptions = new PrinterCounterStatus(
                  printerName);
            printerOptions.GetPrinterCounterStatus();
         }
         else if (arg.equalsIgnoreCase("-ss"))
         {
            System.out.println("Printer Supplies Status Demo");
            PrinterSuppliesStatus printerSuppliesStatus = new PrinterSuppliesStatus(
                  printerName);
            printerSuppliesStatus.GetPrinterSuppliesStatus();
         }
         else if (arg.startsWith("-sc"))
         {
            System.out.format(
                  "Smart Card Demo\n\noptions ---------------------\nmagstripe encode: %b\nprint: %b\nisLandscape: %b\nisDuplex: %b\nisSingleWire: %b\n",
                  (arg.length() > 3 && arg.contains("me")),
                  (arg.length() > 3 && arg.contains("p")), arg.contains("l"),
                  arg.contains("d"), arg.contains("sw"));
            SmartCard smartCard = new SmartCard(printerName);
            smartCard.DemoSmartCard((arg.length() > 3 && arg.contains("me")),
                  (arg.length() > 3 && arg.contains("p")), arg.contains("l"),
                  arg.contains("d"), arg.contains("sw"));
         }
         else if (arg.startsWith("-p"))
         {
            String tempArg = arg.substring(2);

            System.out.format(
                  "PrintCard Demo\n\noptions ---------------------\nisLandscape: %b\nisDuplex: %b\nback side encode: %b\nJIS magnetic: %b\ncopy count: %d\n",
                  (tempArg.contains("l")), (tempArg.contains("d")),
                  (tempArg.contains("e")), tempArg.contains("j"),
                  getCopyCount(arg));
            System.out.println();
            JavaPrint javaPrint = new JavaPrint(printerName,
                  tempArg.contains("d"), tempArg.contains("l"),
                  tempArg.contains("e"), tempArg.contains("e") // block magstripe
                  , false, getCopyCount(arg));
            javaPrint.PrintDemo();
         }
         else if (arg.equalsIgnoreCase("-cAll"))
         {
            System.out.println("CancelAllJobs demo");
            Util.CancelAllJobs(printerName);
         }
         else
         {
            usage();
            System.exit(0);
         }
      }

      // not complete parameters
      if (i < 3)
      {
         usage();
         System.exit(0);
      }

      System.out.println("\nEnd of Demo.");
   }

   public static void usage()
   {
      System.err.println();
      System.err.println("   Demonstrates java functionality of the printer and driver.");
      System.err.println("Notes:");
      System.err.println("> Uses hard-coded data for printing, magnetic stripe, topcoat region and print");
      System.err.println("  blocking region.");
      System.err.println("> Honors the driver printing preferences when options are not set to override.");
      System.err.println("> Default orientation is portrait");
      System.err.println("> The [e] option also demonstrates how topcoat and print blocking escapes are ");
      System.err.println("  used to protect the region over the backside ISO 3-track magnetic stripe.");
      System.err.println();
      System.err.println("  java -jar dxp01Java_SDK.jar -n <printername> <function>[options]");
      System.err.println();
      System.err.println("      -n <printername>. Required. Try -n \"xps card printer\"");
      System.err.println();
      System.err.println("      <function>                        Valid options");
      System.err.println("      -p Print                          [l][d][e][j][number]");
      System.err.println("      -mr Magnetic stripe read          [p][l][d]");
      System.err.println("      -me Magnetic stripe encode        [mr][p][l][d][j]");
      System.err.println("      -sc Park a smart card             [me][p][l][d][sw]");
      System.err.println("      -s  Get printer information 2     [no options]");
      System.err.println("      -cc Get printer card counts 2     [no options]");
      System.err.println("      -ss Get printer supplies status   [no options]");
      System.err.println("      -cAll Cancel all printer Jobs     [no options]");
      System.err.println();
      System.err.println("      [Options]");
      System.err.println("       j  (JIS magnetic)");
      System.err.println("       p  (print sample text)");
      System.err.println("       l  (use landscape orientation (both sides))");
      System.err.println("       d  (print a two-sided (duplex) card)");
      System.err.println("       number(copy count)");
      System.err.println("       e  (encode back-side ISO 3-track magnetic stripe using escapes)");
      System.err.println("       e j(encode back-side JIS track 3 magnetic stripe using magnetic font)");
      System.err.println("       mr (read back-side ISO 3-track magnetic stripe using interactive mode)");
      System.err.println("       me (encode back-side ISO 3-track magnetic stripe using interactive mode)");
      System.err.println("       sw (Singlewire smart card demo)");
      System.err.println();
      System.err.println("  Hard coded values in sample");
      System.err.println("       -me  function: ");
      System.err.println("            track1: \" !#$%&\'\"()*+,-./0123456789:;<=>@ABCDEFGHIJKLMNOPQRSTUVWXYZ");
      System.err.println("                    [\\]^_ !#$%&\'()*+,");
      System.err.println("            track2: 0123456789:;<=>0123456789:;<=>0123456");
      System.err.println("            track3: 0123456789:;<=>0123456789:;... repeat to max length");
      System.err.println();
      System.err.println("       [e]  option:");
      System.err.println("            track1: ABC 123, track2: 345, track3: 789");
      System.err.println();
      System.err.println("       [me] option:");
      System.err.println("            track1: SMARTCARD MAGNETIC STRIPE ENCODE, track2: 123, track3: 678");
      System.err.println("---------------------------------------------------------");
      System.err.println("Examples: (Assume printer name is \"XPS Card Printer\".)");
      System.err.println();
      System.err.println("print 2-sided landscape 2 copies.");
      System.err.println("   java -jar dxp01Java_SDK.jar -n \"XPS Card Printer\" -pld2");
      System.err.println();
      System.err.println("print 2-sided portrait and encode magnetic stripe using escapes.");
      System.err.println("   java -jar dxp01Java_SDK.jar -n \"XPS Card Printer\" -pde");
      System.err.println();
      System.err.println("Interactive magnetic stripe encode, followed by read, followed by printing");
      System.err.println("a two-sided landscape card.");
      System.err.println("   java -jar dxp01Java_SDK.jar -n \"XPS Card Printer\" -memrpld");
      System.err.println();
      System.err.println("Interactive smart card park, followed by resume, followed by interactive");
      System.err.println("magnetic stripe encode, followed by printing a two-sided landscape card.");
      System.err.println("   java -jar dxp01Java_SDK.jar -n \"XPS Card Printer\" -scmepld");
      System.err.println();
      System.err.println();

   }

   private static int getCopyCount(String line)
   {
      int copyCount = 1;
      try
      {
         copyCount = Integer.parseInt(line.substring(line.length() - 1));
      }
      catch (NumberFormatException e)
      {
         copyCount = 1;
      }
      return copyCount;
   }

   private static void extractDll(String path, String name)
   {
      name = name + ".dll";
      try
      {
         String resoucePath = path + "/" + name;
         URL url = ClassLoader.getSystemResource(resoucePath);
         if (url == null)
         {
            System.out.println("unable to locate file: " + resoucePath);
            System.exit(1);
         }

         InputStream in = ClassLoader.getSystemResourceAsStream(resoucePath);
         OutputStream os = new FileOutputStream(name);

         byte[] buffer = new byte[BUFFSIZE];
         int length;
         while ((length = in.read(buffer)) > 0)
         {
            os.write(buffer, 0, length);
         }
         os.close();
         in.close();
      }
      catch (Exception e)
      {
         System.out.format("extractDll() error: %s. Reason: %s", INTEROPDLL,
               e.getMessage());
         System.exit(0);
      }
   }
}
