''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' Copyright (c) Datacard Corporation.  All Rights Reserved.
'
' xps card printer SDK: vb.net 'card completion' sample
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Imports System.Drawing
Imports System.Drawing.Printing
Imports System.IO
Imports System.Threading
Imports dxp01sdk

Class CommandLineOptions
    Public printerName As String = String.Empty
End Class

Class CardCompletionDocument : Inherits PrintDocument
    Public Sub New(commandLineOptions As CommandLineOptions)
        DocumentName = "card completion document"
        PrinterSettings.PrinterName = commandLineOptions.printerName
        PrintController = New StandardPrintController()
        ' suppress annoying progress popups
        AddHandler PrintPage, New PrintPageEventHandler(AddressOf OnPrintPage)
    End Sub

    Private Overloads Sub OnPrintPage(sender As Object, pageEventArgs As PrintPageEventArgs)
        Using font As New Font("Arial", 8)
            Using brush As New SolidBrush(Color.Black)
                Using pen As New Pen(Color.DarkBlue)
                    pageEventArgs.Graphics.DrawString("Sample Text", font, brush, 50, 50)
                    pageEventArgs.Graphics.DrawRectangle(pen, 50, 70, 20, 25)
                End Using
            End Using
        End Using
    End Sub
End Class

Class card_completion
    Private Shared Sub Usage()
        Dim thisExeName As String = Path.GetFileName(Environment.GetCommandLineArgs()(0))
        Console.WriteLine(thisExeName & " demonstrates the card completion for job")
        Console.WriteLine("printed using Windows printing methods.")
        Console.WriteLine()
        Console.WriteLine(thisExeName & " -n <printername> ")
        Console.WriteLine()
        Console.WriteLine("options:")
        Console.WriteLine("  -n <printername>. Required. Try -n ""XPS Card Printer"".")
        Console.WriteLine()
        Environment.Exit(-1)
    End Sub

    Private Shared Function GetCommandlineOptions(args As String()) As CommandLineOptions
        If args.Length = 0 Then
            Usage()
        End If

        Dim commandLineOptions As New CommandLineOptions()
        Dim arguments As New CommandLine.Utility.Arguments(args)

        If String.IsNullOrEmpty(arguments("n")) Then
            Usage()
        End If
        ' we might have a -n with no printer name:
        Dim boolVal As Boolean = False
        If [Boolean].TryParse(arguments("n"), boolVal) Then
            Usage()
        End If
        commandLineOptions.printerName = arguments("n")

        Return commandLineOptions
    End Function

    Private Shared Sub CancelJob(bidiSpl As BidiSplWrap, printerJobID As Integer, errorCode As Integer)
        Dim xmlFormat As String = Strings.PRINTER_ACTION_XML
        Dim input As String = String.Format(xmlFormat, CInt(Actions.Cancel), printerJobID, errorCode)
        bidiSpl.SetPrinterData(Strings.PRINTER_ACTION, input)
    End Sub

    Private Shared Sub PollForJobCompletion(bidiSpl As BidiSplWrap, printerJobID As Integer)
        While True
            Dim temp As String = Strings.JOB_STATUS_XML
            Dim jobInfoXML As String = String.Format(temp, printerJobID)
            Dim jobStatusXML As String = bidiSpl.GetPrinterData(Strings.JOB_STATUS, jobInfoXML)
            Dim jobStatusValues As JobStatusValues = Util.ParseJobStatusXML(jobStatusXML)
            Console.WriteLine(("Printer job id: " & printerJobID & " ") + jobStatusValues._jobState)
            If Strings.JOB_ACTIVE <> jobStatusValues._jobState Then
                Return
            End If
            Thread.Sleep(1000)
        End While
    End Sub

    ''' <summary>
    ''' repeatedly get the printer status until with printer status contains
    ''' a valid printer jobID.
    ''' </summary>
    Private Shared Function WaitForPrinterJobID(bidiSpl As BidiSplWrap, printerName As String) As Integer
        Const sleepMilliSeconds As Integer = 500

        Dim printerStatusXML As String = bidiSpl.GetPrinterData(Strings.PRINTER_MESSAGES)
        Dim printerStatus As PrinterStatusValues = Util.ParsePrinterStatusXML(printerStatusXML)

        Do
            If 0 <> printerStatus._printerJobID Then
                Return printerStatus._printerJobID
            End If

            Console.WriteLine(String.Format("waiting {0} millseconds for printer jobID", sleepMilliSeconds))
            Thread.Sleep(500)

            printerStatusXML = bidiSpl.GetPrinterData(Strings.PRINTER_MESSAGES)

            printerStatus = Util.ParsePrinterStatusXML(printerStatusXML)
        Loop While 0 = printerStatus._errorCode

        Console.WriteLine("printer error:")
        Console.WriteLine(printerStatusXML)

        Return printerStatus._printerJobID
    End Function

    Shared Sub Main(args As String())
        Dim commandLineOptions As CommandLineOptions = GetCommandlineOptions(args)

        Dim bidiSpl As New BidiSplWrap()
        bidiSpl.BindDevice(commandLineOptions.printerName)

        ' optional:
        Dim sdkVersion As String = bidiSpl.GetPrinterData(Strings.SDK_VERSION)
        Console.WriteLine(vbLf & "SDK Version: " & Util.ParseSDKVersionXML(sdkVersion) & vbLf)

        ' optional: is printer is ready for an Ibidi StartJob()?
        Dim printerOptionsXML As String = bidiSpl.GetPrinterData(Strings.PRINTER_OPTIONS2)
        Dim printerOptionsValues As PrinterOptionsValues = Util.ParsePrinterOptionsXML(printerOptionsXML)
        If "Ready" <> printerOptionsValues._printerStatus Then
            Console.WriteLine(commandLineOptions.printerName & " is not ready.")
            bidiSpl.UnbindDevice()
            Environment.Exit(-1)
        End If

        Dim printDocument As CardCompletionDocument = New CardCompletionDocument(commandLineOptions)
        printDocument.Print()

        ' get the printer job id:
        Dim printerJobID As Integer = WaitForPrinterJobID(bidiSpl, commandLineOptions.printerName)

        ' poll for job completion:
        PollForJobCompletion(bidiSpl, printerJobID)

        bidiSpl.UnbindDevice()
    End Sub
End Class
