''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' Copyright (c) Datacard Corporation.  All Rights Reserved.
'
' xps card printer SDK: vb.net 'emboss_indent' sample
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Imports System.Drawing
Imports System.Drawing.Printing
Imports System.IO
Imports System.Threading
Imports dxp01sdk

Class CommandLineOptions
    Public printerName As String = String.Empty
    Public inputHopper As String = "1"
    Public pollForJobCompletion As Boolean = False
End Class

Class EmbossIndenntDocument : Inherits PrintDocument
    Public Sub New(commandLineOptions As CommandLineOptions)
        DocumentName = "emboss indent document"
        PrinterSettings.PrinterName = commandLineOptions.printerName
        PrintController = New StandardPrintController()
        ' suppress annoying progress popups
        AddHandler PrintPage, New PrintPageEventHandler(AddressOf OnPrintPage)
    End Sub

    Private Overloads Sub OnPrintPage(sender As Object, pageEventArgs As PrintPageEventArgs)
        Dim text As String = String.Empty

        ' emit all emboss + indent data driver escape strings. the font and 
        ' brush are not significant; just needed for the DrawString() call.
        Using font As New Font("Arial", 8)
            Using brush As New SolidBrush(Color.Black)
                text = "~EM%1;301;860;Font 11111"
                pageEventArgs.Graphics.DrawString(text, font, brush, 50, 50)

                text = "~EM%2;1600;860;222222"
                pageEventArgs.Graphics.DrawString(text, font, brush, 50, 50)

                text = "~EM%3;301;1460;333333"
                pageEventArgs.Graphics.DrawString(text, font, brush, 50, 50)

                text = "~EM%4;301;1180;444444"
                pageEventArgs.Graphics.DrawString(text, font, brush, 50, 50)

                text = "~EM%5;301;690;555555"
                pageEventArgs.Graphics.DrawString(text, font, brush, 50, 50)

                text = "~EM%6;1600;690;666666"
                pageEventArgs.Graphics.DrawString(text, font, brush, 50, 50)

                text = "~EM%7;301;650;777777"
                pageEventArgs.Graphics.DrawString(text, font, brush, 50, 50)

                text = "~EM%8;301;1000;888888"
                pageEventArgs.Graphics.DrawString(text, font, brush, 50, 50)

                text = "~EM%9;301;1050;999999"
                pageEventArgs.Graphics.DrawString(text, font, brush, 50, 50)

                text = "~EM%10;1600;1050;10 10 10"
                pageEventArgs.Graphics.DrawString(text, font, brush, 50, 50)
            End Using
        End Using
    End Sub
End Class

Class emboss_indent
    Private Shared Sub Usage()
        Dim thisExeName As String = Path.GetFileName(Environment.GetCommandLineArgs()(0))
        Console.WriteLine(thisExeName & " demonstrates the emboss and indent escapes, input hopper")
        Console.WriteLine("selection, and the checking of print ribbon and emboss supplies.")
        Console.WriteLine()
        Console.WriteLine("Emboss and indent data is hard coded.")
        Console.WriteLine()
        Console.WriteLine(thisExeName & " -n <printername> [-i <input hopper>] [-c]")
        Console.WriteLine()
        Console.WriteLine("options:")
        Console.WriteLine("  -n <printername>. Required. Try -n ""XPS Card Printer"".")
        Console.WriteLine("  -i <input hopper>. Optional. Defaults to ""1"".")
        Console.WriteLine("  -c Poll for job status. Optional. Defaults to false.")
        Console.WriteLine()
        Environment.Exit(-1)
    End Sub

    Private Shared Function GetCommandlineOptions(args As String()) As CommandLineOptions
        If args.Length = 0 Then
            Usage()
        End If

        Dim commandLineOptions As New CommandLineOptions()
        Dim arguments As New CommandLine.Utility.Arguments(args)

        If String.IsNullOrEmpty(arguments("n")) Then
            Usage()
        End If
        ' we might have a -n with no printer name:
        Dim boolVal As Boolean = False
        If [Boolean].TryParse(arguments("n"), boolVal) Then
            Usage()
        End If
        commandLineOptions.printerName = arguments("n")

        If Not String.IsNullOrEmpty(arguments("i")) Then
            ' we might have a -i with no input hopper:
            If [Boolean].TryParse(arguments("i"), boolVal) Then
                Usage()
            End If
            commandLineOptions.inputHopper = arguments("i")
        End If

        commandLineOptions.pollForJobCompletion = Not String.IsNullOrEmpty(arguments("c"))
        Return commandLineOptions
    End Function

    Private Shared Sub CancelJob(bidiSpl As BidiSplWrap, printerJobID As Integer, errorCode As Integer)
        Dim xmlFormat As String = strings.PRINTER_ACTION_XML
        Dim input As String = String.Format(xmlFormat, CInt(Actions.Cancel), printerJobID, errorCode)
        bidiSpl.SetPrinterData(strings.PRINTER_ACTION, input)
    End Sub

    Private Shared Sub PollForJobCompletion(bidiSpl As BidiSplWrap, printerJobID As Integer)
        While True
            Dim temp As String = strings.JOB_STATUS_XML
            Dim jobInfoXML As String = String.Format(temp, printerJobID)
            Dim jobStatusXML As String = bidiSpl.GetPrinterData(strings.JOB_STATUS, jobInfoXML)
            Dim jobStatusValues As JobStatusValues = Util.ParseJobStatusXML(jobStatusXML)
            Console.WriteLine(("Printer job id: " & printerJobID & " ") + jobStatusValues._jobState)
            If strings.JOB_ACTIVE <> jobStatusValues._jobState Then
                Return
            End If
            Thread.Sleep(1000)
        End While
    End Sub

    ''' <summary>
    ''' for this sample, pass the hopper name from the command line into the
    ''' IBidi StartJob() call.
    ''' </summary>
    ''' <param name="inputHopper">
    ''' a string like "1", "Exception", or empty string
    ''' </param>
    ''' <returns>
    ''' the printer Job ID, an integer
    ''' </returns>
    Private Shared Function StartJob(bidiSpl As BidiSplWrap, inputHopper As String) As Integer
        ' zero means "do not check print ribbon"
        Dim startJobXml As String = String.Format(strings.STARTJOB_XML, inputHopper, 0, 0)
        ' zero means "do not check emboss supplies"
        Dim printerStatusXML As String = bidiSpl.SetPrinterData(strings.STARTJOB, startJobXml)
        Dim printerStatusValues As PrinterStatusValues = Util.ParsePrinterStatusXML(printerStatusXML)
        If 0 <> printerStatusValues._errorCode Then
            Console.WriteLine("StartJob() fail; canceling the job. Printer Error: " + printerStatusValues._errorString)
            CancelJob(bidiSpl, printerStatusValues._printerJobID, printerStatusValues._errorCode)
            bidiSpl.UnbindDevice()
            Environment.Exit(-1)
        End If
        Return printerStatusValues._printerJobID
    End Function

    Shared Sub Main(args As String())
        Dim commandLineOptions As CommandLineOptions = GetCommandlineOptions(args)

        Dim bidiSpl As New BidiSplWrap()
        bidiSpl.BindDevice(commandLineOptions.printerName)

        ' optional:
        Dim sdkVersion As String = bidiSpl.GetPrinterData(strings.SDK_VERSION)
        Console.WriteLine(vbLf & "SDK Version: " & Util.ParseSDKVersionXML(sdkVersion) & vbLf)

        ' optional: is printer is ready for an Ibidi StartJob()?
        Dim printerOptionsXML As String = bidiSpl.GetPrinterData(strings.PRINTER_OPTIONS2)
        Dim printerOptionsValues As PrinterOptionsValues = Util.ParsePrinterOptionsXML(printerOptionsXML)
        If "Ready" <> printerOptionsValues._printerStatus Then
            Console.WriteLine(commandLineOptions.printerName & " is not ready.")
            bidiSpl.UnbindDevice()
            Environment.Exit(-1)
        End If

        Dim printerJobID As Integer = StartJob(bidiSpl, commandLineOptions.inputHopper)

        Dim printDocument As EmbossIndenntDocument = New EmbossIndenntDocument(commandLineOptions)
        printDocument.Print()
        Util.WaitForWindowsJobID(bidiSpl, commandLineOptions.printerName)

        bidiSpl.SetPrinterData(strings.ENDJOB)

        If commandLineOptions.pollForJobCompletion Then
            PollForJobCompletion(bidiSpl, printerJobID)
        End If

        bidiSpl.UnbindDevice()
    End Sub
End Class
