﻿''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' Copyright (c) Datacard Corporation.  All Rights Reserved.
'
' xps printer sdk vb.net magstripe sample.
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Imports System.IO
Imports System.Threading
Imports dxp01sdk

Public Class CommandLineOptions
    Public printerName As String
    Public readMagstripe As Boolean
    Public print As Boolean
    Public pollForJobCompletion As Boolean
    Public jisRequest As Boolean
    Public verbose As Boolean
End Class

Class magstripe
    Private Shared Sub usage()
        Console.WriteLine("Magstripe.exe demonstrates interactive mode magnetic stripe encoding with")
        Console.WriteLine("  options to read the magnetic stripe data, print, and poll for job")
        Console.WriteLine("  completion status.")
        Console.WriteLine()
        Console.WriteLine("> Uses hard-coded data for magnetic stripe and printing.")
        Console.WriteLine()
        Console.WriteLine("magstripe.exe -n <printername> [-r] [-p] [-c] [-j] [-v]")
        Console.WriteLine()
        Console.WriteLine("options:")
        Console.WriteLine("  -n <printername>. Required. Try -n ""XPS Card Printer"".")
        Console.WriteLine("  -r Reads back the encoded magnetic stripe data.")
        Console.WriteLine("  -p Print simple black text on front side of card. ")
        Console.WriteLine("  -c Poll for job status. ")
        Console.WriteLine("  -j JIS magnetic. ")
        Console.WriteLine("  -v Display detailed information with timings.")
        Console.WriteLine()
        Console.WriteLine("magstripe.exe -n ""XPS Card Printer""")
        Console.WriteLine("  Encodes data on all three tracks of an ISO 3-track magnetic stripe on the")
        Console.WriteLine("  backside of a card.")
        Console.WriteLine()
        Console.WriteLine("magstripe -n ""XPS Card Printer"" -r -p -c")
        Console.WriteLine("  Encodes data on all three tracks of an ISO 3-track magnetic stripe on the")
        Console.WriteLine("  backside of a card, reads the data from the magnetic stripe and displays")
        Console.WriteLine("  it, prints black text on the front side of the card, and polls and")
        Console.WriteLine("  displays job status.")
        Environment.Exit(-1)
    End Sub

    Private Shared Function GetCommandlineOptions(args As String()) As CommandLineOptions
        If args.Length = 0 Then
            usage()
        End If

        Dim commandLineOptions As New CommandLineOptions()
        Dim arguments As New CommandLine.Utility.Arguments(args)

        If Not String.IsNullOrEmpty(arguments("h")) Then
            usage()
        End If

        If String.IsNullOrEmpty(arguments("n")) Then
            Console.WriteLine("printer name is required")
            Environment.Exit(-1)
        Else
            ' we might have a -n with no printer name:
            Dim boolVal As Boolean = False
            If [Boolean].TryParse(arguments("n"), boolVal) Then
                Console.WriteLine("printer name is required")
                Environment.Exit(-1)
            End If
            commandLineOptions.printerName = arguments("n")
        End If

        commandLineOptions.readMagstripe = Not String.IsNullOrEmpty(arguments("r"))
        commandLineOptions.print = Not String.IsNullOrEmpty(arguments("p"))
        commandLineOptions.pollForJobCompletion = Not String.IsNullOrEmpty(arguments("c"))
        commandLineOptions.jisRequest = Not String.IsNullOrEmpty(arguments("j"))
        commandLineOptions.verbose = Not String.IsNullOrEmpty(arguments("v"))

        Return commandLineOptions
    End Function

    Private Shared Sub ResumeJob(bidiSpl As BidiSplWrap, printerJobID As Integer, errorCode As Integer)
        Dim xmlFormat As String = strings.PRINTER_ACTION_XML
        Dim input As String = String.Format(xmlFormat, CInt(Actions.[Resume]), printerJobID, errorCode)
        bidiSpl.SetPrinterData(strings.PRINTER_ACTION, input)
    End Sub

    Private Shared Sub CancelJob(bidiSpl As BidiSplWrap, printerJobID As Integer, errorCode As Integer, verbose As Boolean)
        If verbose Then
            Console.WriteLine(Util.GetCurrentTimeString() & " CancelJob() start")
        End If

        Dim xmlFormat As String = strings.PRINTER_ACTION_XML
        Dim input As String = String.Format(xmlFormat, CInt(Actions.Cancel), printerJobID, errorCode)
        bidiSpl.SetPrinterData(strings.PRINTER_ACTION, input)

        If verbose Then
            Console.WriteLine(Util.GetCurrentTimeString() & " CancelJob() return")
        End If
    End Sub

    Private Shared Sub PollForJobCompletion(bidiSpl As BidiSplWrap, printerJobID As Integer, verbose As Boolean)
        If verbose Then
            Console.WriteLine(Util.GetCurrentTimeString() & " PollForJobCompletion() start")
        End If

        While True
            Dim printerStatusXML As String = bidiSpl.GetPrinterData(strings.PRINTER_MESSAGES)
            Dim printerStatusValues As PrinterStatusValues = Util.ParsePrinterStatusXML(printerStatusXML)
            If 0 <> printerStatusValues._errorCode Then
                Console.WriteLine(printerStatusValues._errorString + " severity: " & printerStatusValues._errorSeverity.ToString())
                Console.WriteLine("canceling printer jobID " & printerStatusValues._printerJobID.ToString())
                CancelJob(bidiSpl, printerStatusValues._printerJobID, printerStatusValues._errorCode, verbose)
                Return
            End If

            Dim xmlFormat As String = strings.JOB_STATUS_XML
            Dim jobInfoXML As String = String.Format(xmlFormat, printerJobID)
            Dim jobStatusXML As String = bidiSpl.GetPrinterData(strings.JOB_STATUS, jobInfoXML)
            Dim jobStatusValues As JobStatusValues = Util.ParseJobStatusXML(jobStatusXML)

            Console.WriteLine(("Printer job id: " & printerJobID & " ") + jobStatusValues._jobState)

            If strings.JOB_ACTIVE <> jobStatusValues._jobState Then
                Return
            End If
            Thread.Sleep(1000)
        End While
    End Sub

    Public Shared Sub ReadMagstripe(bidiSpl As BidiSplWrap, jisRequest As Boolean, verbose As Boolean)
        If verbose Then
            Console.WriteLine(Util.GetCurrentTimeString() & " ReadMagstripe() caling bidi GetPrinterData()")
        End If

        ' replace schema string MAGSTRIPE_READ to MAGSTRIPE_READ_FRONT for front side read
        Dim printerStatusXML As String = bidiSpl.GetPrinterData(strings.MAGSTRIPE_READ)

        If verbose Then
            Console.WriteLine(Util.GetCurrentTimeString() & " GetPrinterData() return")
        End If

        Dim printerStatusValues As PrinterStatusValues = Util.ParsePrinterStatusXML(printerStatusXML)
        If 0 = printerStatusValues._errorCode Then
            Console.WriteLine("Magstripe data read successfully; printer job id: " + Str(printerStatusValues._printerJobID))

            Dim track1 As String = ""
            Dim track2 As String = ""
            Dim track3 As String = ""

            Util.ParseMagstripeStrings(printerStatusXML, track1, track2, track3, jisRequest)

            If track1.Length <> 0 Then
                ' Convert the Base64 UUEncoded output.
                Dim binaryData As Byte() = System.Convert.FromBase64String(track1)
                Dim str As String = System.Text.Encoding.UTF8.GetString(binaryData)
                Console.WriteLine(" track1 Base64 decoded: " & str)
            End If

            If track2.Length <> 0 Then
                ' Convert the Base64 UUEncoded output.
                Dim binaryData As Byte() = System.Convert.FromBase64String(track2)
                Dim str As String = System.Text.Encoding.UTF8.GetString(binaryData)
                Console.WriteLine(" track2 Base64 decoded: " & str)
            End If

            If track3.Length <> 0 Then
                ' Convert the Base64 UUEncoded output.
                Dim binaryData As Byte() = System.Convert.FromBase64String(track3)
                Dim str As String = System.Text.Encoding.UTF8.GetString(binaryData)
                Console.WriteLine(" track3 Base64 decoded: " & str)
            End If
        Else
            Console.WriteLine("ReadMagstripe() fail; canceling the job. Printer Error: " + printerStatusValues._errorString)
            CancelJob(bidiSpl, printerStatusValues._printerJobID, printerStatusValues._errorCode, verbose)
            bidiSpl.UnbindDevice()
            Environment.Exit(-1)
        End If
    End Sub

    Public Shared Sub EncodeMagstripe(bidiSpl As BidiSplWrap, jisRequest As Boolean, verbose As Boolean)
        ' Hardcoded XML to encode all 3 tracks in IAT mode.
        ' track 1 = "TRACK1", track 2 = "1122", track 3 = "321"
        Dim trackDataXML As String = "<?xml version=""1.0"" encoding=""utf-8""?>" & "<magstripe>" & "<track number=""1""><base64Data>VFJBQ0sx</base64Data></track>" & "<track number=""2""><base64Data>MTEyMg==</base64Data></track>" & "<track number=""3""><base64Data>MzIx</base64Data></track>" & "</magstripe>"

        If jisRequest Then
            ' JIS only allows track 3 = "321"
            trackDataXML = "<?xml version=""1.0"" encoding=""utf-8""?>" & "<magstripe>" & "<track number=""1""><base64Data></base64Data></track>" & "<track number=""2""><base64Data></base64Data></track>" & "<track number=""3""><base64Data>MzIx</base64Data></track>" & "</magstripe>"
        End If

        If verbose Then
            Console.WriteLine(Util.GetCurrentTimeString() & " EncodeMagstripe() caling bidi SetPrinterData()")
        End If

        ' replace schema string MAGSTRIPE_ENCODE to MAGSTRIPE_ENCODE_FRONT for front side encode
        Dim printerStatusXML As String = bidiSpl.SetPrinterData(strings.MAGSTRIPE_ENCODE, trackDataXML)

        If verbose Then
            Console.WriteLine(Util.GetCurrentTimeString() & " SetPrinterData() return")
        End If

        Dim printerStatusValues As PrinterStatusValues = Util.ParsePrinterStatusXML(printerStatusXML)
        If 0 <> printerStatusValues._errorCode Then
            Console.WriteLine("EncodeMagstripe() fail; canceling the job. Printer Error: " + printerStatusValues._errorString)
            CancelJob(bidiSpl, printerStatusValues._printerJobID, printerStatusValues._errorCode, verbose)
            bidiSpl.UnbindDevice()
            Environment.Exit(-1)
        End If

        If verbose Then
            Console.WriteLine(Util.GetCurrentTimeString() & " EncodeMagstripe() return")
        End If
    End Sub

    Private Shared Sub StartJob(bidiSpl As BidiSplWrap, ByRef printerJobID As Integer, verbose As Boolean)
        If verbose Then
            Console.WriteLine(Util.GetCurrentTimeString() & " StartJob() begin")
        End If

        ' use default input hopper
        ' zero means "do not check print ribbon"
        ' zero means "do not check emboss supplies"
        Dim startJobXml As String = String.Format(strings.STARTJOB_XML, "", 0, 0)

        Dim printerStatusXML As String = bidiSpl.SetPrinterData(strings.STARTJOB, startJobXml)

        Dim printerStatusValues As PrinterStatusValues = Util.ParsePrinterStatusXML(printerStatusXML)
        If 0 <> printerStatusValues._errorCode Then
            Console.WriteLine("StartJob() fail; canceling the job. Printer Error: " + printerStatusValues._errorString)
            CancelJob(bidiSpl, printerStatusValues._printerJobID, printerStatusValues._errorCode, verbose)
            bidiSpl.UnbindDevice()
            Environment.Exit(-1)
        Else
            printerJobID = printerStatusValues._printerJobID
        End If

        If verbose Then
            Console.WriteLine(Util.GetCurrentTimeString() & " StartJob() return")
        End If
    End Sub

    ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    ' Main()
    ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    Public Shared Sub Main(ByVal args As String())
        Dim printerJobID As Integer = 0
        Dim commandLineOptions As CommandLineOptions = GetCommandlineOptions(args)

        Dim bidiSpl As New BidiSplWrap()
        bidiSpl.BindDevice(commandLineOptions.printerName)

        If commandLineOptions.verbose Then
            Console.WriteLine(Util.GetCurrentTimeString() & " calling GetPrinterData(version)")
        End If

        Dim sdkVersionXml As String = bidiSpl.GetPrinterData(strings.SDK_VERSION)

        If commandLineOptions.verbose Then
            Console.WriteLine(Util.GetCurrentTimeString() & " GetPrinterData(version) return")
        End If

        ' Check if printer is ready to accept magstripe job
        Dim printerOptionsXML As String = bidiSpl.GetPrinterData(strings.PRINTER_OPTIONS2)
        Dim printerOptionsValues As PrinterOptionsValues = Util.ParsePrinterOptionsXML(printerOptionsXML)
        If "Ready" <> printerOptionsValues._printerStatus Then
            Console.WriteLine(commandLineOptions.printerName & " is not ready.")
            bidiSpl.UnbindDevice()
            Environment.Exit(-1)
        End If

        ' Start the magstripe job
        StartJob(bidiSpl, printerJobID, commandLineOptions.verbose)

        ' Encode the magstripe
        EncodeMagstripe(bidiSpl, commandLineOptions.jisRequest, commandLineOptions.verbose)

        ' Check the command line parameters if user wants to read the magstripe
        If commandLineOptions.readMagstripe Then
            ' Read the magstripe
            ReadMagstripe(bidiSpl, commandLineOptions.jisRequest, commandLineOptions.verbose)
        End If

        ' Check if user wants print
        If commandLineOptions.print Then
            If commandLineOptions.verbose Then
                Console.WriteLine(Util.GetCurrentTimeString() & " calling PrintTextAndGraphics()")
            End If

            Util.PrintTextAndGraphics(bidiSpl, commandLineOptions.printerName)

            If commandLineOptions.verbose Then
                Console.WriteLine(Util.GetCurrentTimeString() & " PrintTextAndGraphics() returned")
            End If
        End If


        If commandLineOptions.verbose Then
            Console.WriteLine(Util.GetCurrentTimeString() & " calling EndJob()")
        End If

        ' End the magstripe job
        bidiSpl.SetPrinterData(strings.ENDJOB)

        If commandLineOptions.verbose Then
            Console.WriteLine(Util.GetCurrentTimeString() & " EndJob() returned")
        End If

        ' Check if user wants to poll the job for completion status
        If commandLineOptions.pollForJobCompletion Then
            PollForJobCompletion(bidiSpl, printerJobID, commandLineOptions.verbose)
        End If

        If commandLineOptions.verbose Then
            Console.WriteLine(Util.GetCurrentTimeString() & " calling UnbindDevice()")
        End If

        bidiSpl.UnbindDevice()

        If commandLineOptions.verbose Then
            Console.WriteLine(Util.GetCurrentTimeString() & " UnbindDevice() returned")
        End If
    End Sub
End Class
