﻿''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' Copyright (c) Datacard Corporation.  All Rights Reserved.
'
' xps driver sdk vb.net print sample.
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Imports System.Drawing.Printing
Imports dxp01sdk

Namespace print
    Class print
        Private Shared Sub CancelJob(bidiSpl As BidiSplWrap, printerJobID As Integer, errorCode As Integer)
            Dim xmlFormat As String = strings.PRINTER_ACTION_XML
            Dim input As String = String.Format(xmlFormat, CInt(Actions.Cancel), printerJobID, errorCode)
            bidiSpl.SetPrinterData(strings.PRINTER_ACTION, input)
        End Sub

        ''' <summary>
        ''' issue an IBidi StartJob with xml markup to start an IBidi job, just to have
        ''' the printer check for supplies. If the printer is out of supplies, throw an
        ''' exception.
        ''' </summary>
        ''' <returns>printerJobID</returns>
        Private Shared Function CheckSupplies(bidiSpl As BidiSplWrap, printerOptionsValues As PrinterOptionsValues) As Integer
            Dim checkPrintRibbon As Integer = If((printerOptionsValues._printHead = "Installed"), 1, 0)
            Dim checkEmbossSupplies As Integer = If(printerOptionsValues._moduleEmbosser = "None", 0, 1)

            ' use default input hopper in this sample
            Dim startJobXml As String = String.Format(strings.STARTJOB_XML, "", checkPrintRibbon, checkEmbossSupplies)

            Dim printerStatusXML As String = bidiSpl.SetPrinterData(strings.STARTJOB, startJobXml)
            Dim printerStatusValues As PrinterStatusValues = Util.ParsePrinterStatusXML(printerStatusXML)
            If 0 <> printerStatusValues._errorCode Then
                ' the StartJob failed; send a Cancel(); do NOT send an EndJob():
                CancelJob(bidiSpl, printerStatusValues._printerJobID, printerStatusValues._errorCode)
                Throw New Exception("Canceling the job. Printer Error: " + printerStatusValues._errorString)
            End If

            Return printerStatusValues._printerJobID
        End Function

        Shared Sub Main(args As String())
            Dim commandLineOptions As CommandLineOptions = commandLineOptions.CreateFromArguments(args)
            commandLineOptions.Validate()

            Dim bidiSpl As BidiSplWrap = Nothing
            Try
                bidiSpl = New BidiSplWrap()
                bidiSpl.BindDevice(commandLineOptions.printerName)

                Dim sdkVersion As String = bidiSpl.GetPrinterData(strings.SDK_VERSION)
                Console.WriteLine(Environment.NewLine & "SDK Version: " & Util.ParseSDKVersionXML(sdkVersion) & Environment.NewLine)

                Dim printerOptionsXML As String = bidiSpl.GetPrinterData(strings.PRINTER_OPTIONS2)
                Dim printerOptionsValues As PrinterOptionsValues = Util.ParsePrinterOptionsXML(printerOptionsXML)
                If "Ready" <> printerOptionsValues._printerStatus Then
                    Throw New Exception(commandLineOptions.printerName + " is not ready.")
                End If

                Dim printerJobID As Integer = 0
                If commandLineOptions.checkSupplies Then
                    printerJobID = CheckSupplies(bidiSpl, printerOptionsValues)
                End If

                Dim printDocument As New SamplePrintDocument(commandLineOptions)
                printDocument.PrintController = New StandardPrintController() ' suppress annoying progress popups
                AddHandler printDocument.BeginPrint, New PrintEventHandler(AddressOf printDocument.OnBeginPrint)
                AddHandler printDocument.QueryPageSettings, New QueryPageSettingsEventHandler(AddressOf printDocument.OnQueryPageSettings)
                AddHandler printDocument.PrintPage, New PrintPageEventHandler(AddressOf printDocument.OnPrintPage)
                printDocument.Print()

                If 0 <> printerJobID Then
                    ' we did a StartJob() to check for supplies. So, wait a
                    ' moment for the spooling to finish, and then issue an EndJob():
                    Util.WaitForWindowsJobID(bidiSpl, commandLineOptions.printerName)
                    bidiSpl.SetPrinterData(strings.ENDJOB)
                End If

            Catch ex As Exception
                Console.WriteLine(ex.Message)
            Finally
                bidiSpl.UnbindDevice()
            End Try
        End Sub
    End Class
End Namespace
