﻿''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' Copyright (c) Datacard Corporation.  All Rights Reserved.
'
' xps card printer SDK: vb.net smartcard sample
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Imports System
Imports System.IO
Imports System.Threading
Imports System.Windows.Forms
Imports dxp01sdk

Public Class CommandLineOptions
    Public printerName As String
    Public print As Boolean
    Public pollForJobCompletion As Boolean
End Class

Class smartcard
    Private Shared Sub Usage()
        Console.WriteLine("Smartcard.exe demonstrates interactive mode parking a card in the smart card")
        Console.WriteLine("  station, moving the card from the station, and options to print and poll for")
        Console.WriteLine("  job completion.")
        Console.WriteLine()
        Console.WriteLine("> Uses hard-coded data for printing.")
        Console.WriteLine()
        Console.WriteLine("smartcard.exe -n <printername> [-p] [-c]")
        Console.WriteLine()
        Console.WriteLine("options:")
        Console.WriteLine("  -n <printername>. Required. Try -n ""XPS Card Printer"".")
        Console.WriteLine("  -p Print text and graphic on front side of card.")
        Console.WriteLine("  -c Poll for job status.")
        Console.WriteLine()
        Console.WriteLine("smartcard.exe -n ""XPS Card Printer""")
        Console.WriteLine("  Parks a card in the printer smart card station, asks you to continue or")
        Console.WriteLine("  reject and then does what you requested.")
        Console.WriteLine()
        Console.WriteLine("smartcard.exe -n ""XPS Card Printer"" -p -c")
        Console.WriteLine("  Parks a card in printer smart card station, moves the card from the station,")
        Console.WriteLine("  prints simple text and graphics on the front side of the card, and polls and")
        Console.WriteLine("  displays job status.")
        Environment.Exit(-1)
    End Sub

    Private Shared Function GetCommandlineOptions(ByVal args As String()) As CommandLineOptions
        If args.Length = 0 Then
            Usage()
        End If

        Dim commandLineOptions As New CommandLineOptions()
        Dim arguments As New CommandLine.Utility.Arguments(args)

        If Not String.IsNullOrEmpty(arguments("h")) Then
            Usage()
        End If

        If String.IsNullOrEmpty(arguments("n")) Then
            Console.WriteLine("printer name is required")
            Environment.Exit(-1)
        Else
            ' we might have a -n with no printer name:
            Dim boolVal As Boolean = False
            If [Boolean].TryParse(arguments("n"), boolVal) Then
                Console.WriteLine("printer name is required")
                Environment.Exit(-1)
            End If
            commandLineOptions.printerName = arguments("n")
        End If

        commandLineOptions.print = Not String.IsNullOrEmpty(arguments("p"))
        commandLineOptions.pollForJobCompletion = Not String.IsNullOrEmpty(arguments("c"))

        Return commandLineOptions
    End Function

    Private Shared Sub ResumeJob(ByVal bidiSpl As BidiSplWrap, ByVal printerJobID As Integer, ByVal errorCode As Integer)
        Dim xmlFormat As String = strings.PRINTER_ACTION_XML
        Dim input As String = String.Format(xmlFormat, CInt(Actions.[Resume]), printerJobID, errorCode)
        bidiSpl.SetPrinterData(strings.PRINTER_ACTION, input)
    End Sub

    Private Shared Sub CancelJob(ByVal bidiSpl As BidiSplWrap, ByVal printerJobID As Integer, ByVal errorCode As Integer)
        Dim xmlFormat As String = strings.PRINTER_ACTION_XML
        Dim input As String = String.Format(xmlFormat, CInt(Actions.Cancel), printerJobID, errorCode)
        bidiSpl.SetPrinterData(strings.PRINTER_ACTION, input)
    End Sub

    ''' <summary>
    ''' Simulate a smartcard personalization operation.
    ''' 
    ''' This is where calls to a third-party library would be used to communicate
    ''' with the chip.
    ''' </summary>
    ''' <returns></returns>
    Private Shared Function PersonalizeSmartcard() As Boolean
        Dim caption As String = "A card has been parked in smartcard module"
        Dim message As String = "This is where smartcard personalization happens." & vbCr & vbLf & vbCr & vbLf & "Press 'Yes' to simulate a successful smartcard personalization." & vbCr & vbLf & vbCr & vbLf & "Press 'No' to simulate a failed smartcard personalization."

        Dim result As DialogResult = MessageBox.Show(message, caption, MessageBoxButtons.YesNo)
        Return (result = DialogResult.Yes)
    End Function

    Private Shared Sub PollForJobCompletion(ByVal bidiSpl As BidiSplWrap, ByVal printerJobID As Integer)
        While True
            Dim temp As String = strings.JOB_STATUS_XML
            Dim jobInfoXML As String = String.Format(temp, printerJobID)
            Dim jobStatusXML As String = bidiSpl.GetPrinterData(strings.JOB_STATUS, jobInfoXML)
            Dim jobStatusValues As JobStatusValues = Util.ParseJobStatusXML(jobStatusXML)

            Console.WriteLine(("Printer job id: " & printerJobID & " ") + jobStatusValues._jobState)

            If strings.JOB_ACTIVE <> jobStatusValues._jobState Then
                Return
            End If
            Thread.Sleep(1000)
        End While
    End Sub

    Private Shared Sub StartJob(ByVal bidiSpl As BidiSplWrap, ByRef printerJobID As Integer)
        Dim printerStatusXML As String = bidiSpl.SetPrinterData(strings.STARTJOB)
        Dim printerStatusValues As PrinterStatusValues = Util.ParsePrinterStatusXML(printerStatusXML)
        If 0 <> printerStatusValues._errorCode Then
            Console.WriteLine("StartJob() fail; canceling the job. Printer Error: " + printerStatusValues._errorString)
            CancelJob(bidiSpl, printerStatusValues._printerJobID, printerStatusValues._errorCode)
            bidiSpl.UnbindDevice()
            Environment.Exit(-1)
        Else
            printerJobID = printerStatusValues._printerJobID
        End If
    End Sub

    Private Shared Sub SmartcardPark(ByVal bidiSpl As BidiSplWrap)
        Dim printerStatusXML As String = bidiSpl.SetPrinterData(strings.SMARTCARD_PARK)
        Dim printerStatusValues As PrinterStatusValues = Util.ParsePrinterStatusXML(printerStatusXML)
        If 0 <> printerStatusValues._errorCode Then
            Console.WriteLine("SmartcardPark() fail; canceling the job. Printer Error: " + printerStatusValues._errorString)
            CancelJob(bidiSpl, printerStatusValues._printerJobID, printerStatusValues._errorCode)
            bidiSpl.UnbindDevice()
            Environment.Exit(-1)
        End If
    End Sub

    Private Shared Sub SmartcardPersonalizationFailed(ByVal bidiSpl As BidiSplWrap, ByVal printerJobID As Integer)
        Console.WriteLine("SmartcardPersonalizationFailed(); canceling the job.")
        CancelJob(bidiSpl, printerJobID, 0)
        bidiSpl.UnbindDevice()
        Environment.Exit(-1)
    End Sub

    ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    ' Main()
    ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    Public Shared Sub Main(ByVal args As String())
        Dim commandLineOptions As CommandLineOptions = GetCommandlineOptions(args)

        Dim bidiSpl As New BidiSplWrap()
        bidiSpl.BindDevice(commandLineOptions.printerName)

        ' Get SDK version 
        Dim sdkVersion As String = bidiSpl.GetPrinterData(strings.SDK_VERSION)
        Console.WriteLine(vbLf & "SDK Version: " & Util.ParseSDKVersionXML(sdkVersion) & vbLf)

        ' is printer is ready to accept a smartcard job?
        Dim printerOptionsXML As String = bidiSpl.GetPrinterData(strings.PRINTER_OPTIONS2)
        Dim printerOptionsValues As PrinterOptionsValues = Util.ParsePrinterOptionsXML(printerOptionsXML)
        If "Ready" <> printerOptionsValues._printerStatus Then
            Console.WriteLine(commandLineOptions.printerName & " is not ready.")
            bidiSpl.UnbindDevice()
            Environment.Exit(-1)
        End If

        ' start an IBidiSpl job:
        Dim printerJobID As Integer = 0
        StartJob(bidiSpl, printerJobID)

        ' Park the card at smartcard station:
        SmartcardPark(bidiSpl)

        ' The card has been parked in the smartcard station. Personalize the smart card.
        Dim personalizedOK As Boolean = PersonalizeSmartcard()
        If Not personalizedOK Then
            SmartcardPersonalizationFailed(bidiSpl, printerJobID)
        End If

        ' The Smartcard Personalization succeeded; send Resume command
        ResumeJob(bidiSpl, printerJobID, 0)

        If commandLineOptions.print Then
            Util.PrintTextAndGraphics(bidiSpl, commandLineOptions.printerName)
        End If

        bidiSpl.SetPrinterData(strings.ENDJOB)

        If commandLineOptions.pollForJobCompletion Then
            PollForJobCompletion(bidiSpl, printerJobID)
        End If

        bidiSpl.UnbindDevice()
    End Sub
End Class
