''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' Copyright (c) Datacard Corporation.  All Rights Reserved.
'
' XPS Printer Driver SDK: Single-wire smartcard personalization vb.net sample.
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Imports System.Collections.Generic
Imports System.IO
Imports System.Text
Imports System.Threading
Imports dxp01sdk

Public Class CommandLineOptions
    Public printerName As String
    Public print As Boolean
    Public pollForJobCompletion As Boolean
    Public verbose As Boolean
End Class

Class smartcard_singlewire
    Private Shared Sub Usage()
        Dim thisExeName As String = Path.GetFileName(Environment.GetCommandLineArgs()(0))
        Console.Write(thisExeName)
        Console.WriteLine(" demonstrates interactive mode parking a card in the")
        Console.WriteLine("  smart card station, performing single-wire smartcard functions, moving")
        Console.WriteLine("  the card from the station, and options to print and poll for job completion.")
        Console.WriteLine()
        Console.WriteLine("  Uses hard-coded data for printing.")
        Console.WriteLine()
        Console.WriteLine(thisExeName & " -n <printername> [-p] [-c] [-v]")
        Console.WriteLine()
        Console.WriteLine("options:")
        Console.WriteLine("  -n <printername>. Required. Try -n ""XPS Card Printer"".")
        Console.WriteLine("  -p Print text and graphic on front side of card.")
        Console.WriteLine("  -c Poll for job status.")
        Console.WriteLine("  -v Display detailed information with timings.")
        Console.WriteLine()
        Console.WriteLine(thisExeName + " -n ""XPS Card Printer""")
        Console.WriteLine("  Parks a card in the printer smart card station, asks you to continue or")
        Console.WriteLine("  reject and then does what you requested.")
        Console.WriteLine()
        Console.WriteLine(thisExeName + " -n ""XPS Card Printer"" -p -c")
        Console.WriteLine("  Parks a card in printer smart card station, moves the card from the station,")
        Console.WriteLine("  prints simple text and graphics on the front side of the card, and polls and")
        Console.WriteLine("  displays job status.")
        Environment.Exit(-1)
    End Sub

    Private Shared Function GetCommandlineOptions(args As String()) As CommandLineOptions
        If args.Length = 0 Then
            Usage()
        End If

        Dim commandLineOptions As CommandLineOptions = New CommandLineOptions()
        Dim arguments As CommandLine.Utility.Arguments = New CommandLine.Utility.Arguments(args)

        If String.IsNullOrEmpty(arguments("n")) Then
            Console.WriteLine("printer name is required")
            Environment.Exit(-1)
        Else
            ' we might have a -n with no printer name:
            Dim boolVal As Boolean = False
            If Boolean.TryParse(arguments("n"), boolVal) Then
                Console.WriteLine("printer name is required")
                Environment.Exit(-1)
            End If
            commandLineOptions.printerName = arguments("n")
        End If

        commandLineOptions.print = Not String.IsNullOrEmpty(arguments("p"))
        commandLineOptions.pollForJobCompletion = Not String.IsNullOrEmpty(arguments("c"))
        commandLineOptions.verbose = Not String.IsNullOrEmpty(arguments("v"))

        Return commandLineOptions
    End Function

    Private Shared Sub ResumeJob(bidiSpl As BidiSplWrap, printerJobID As Integer, errorCode As Integer, verbose As Boolean)
        If verbose Then
            Console.WriteLine(Util.GetCurrentTimeString() & " ResumeJob() start")
        End If

        Dim xmlFormat As String = strings.PRINTER_ACTION_XML
        Dim input As String = String.Format(xmlFormat, CInt(Actions.Resume), printerJobID, errorCode)
        Console.WriteLine("issuing Resume after smartcard:")
        bidiSpl.SetPrinterData(strings.PRINTER_ACTION, input)

        If verbose Then
            Console.WriteLine(Util.GetCurrentTimeString() & " ResumeJob() return")
        End If
    End Sub

    Private Shared Sub CancelJob(bidiSpl As BidiSplWrap, printerJobID As Integer, errorCode As Integer, verbose As Boolean)
        If verbose Then
            Console.WriteLine(Util.GetCurrentTimeString() & " CancdlJob() start")
        End If

        Dim xmlFormat As String = strings.PRINTER_ACTION_XML
        Dim input As String = String.Format(xmlFormat, CInt(Actions.Cancel), printerJobID, errorCode)
        bidiSpl.SetPrinterData(strings.PRINTER_ACTION, input)

        If verbose Then
            Console.WriteLine(Util.GetCurrentTimeString() & " CancdlJob() return")
        End If
    End Sub

    ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    ' SCardResultMessage()
    ' format a message for display.
    ' SCARD errors are declared in WinError.H
    ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    Private Shared Function SCardResultMessage(scardResult As Long, message As String) As String
        Dim sb As StringBuilder = New StringBuilder()
        sb.Append(message)
        sb.Append(" result: ")
        sb.Append(scardResult.ToString())
        sb.Append("; ")
        sb.Append(Util.Win32ErrorString(CInt(scardResult)))
        Return sb.ToString()
    End Function

    '''
    ''' <summary>
    ''' a card is in the printer and stated in the smartcard module. The
    ''' SCardConnect() call has succeeded.
    ''' talk to the reader and the chip (contact or contactless).
    ''' </summary>
    ''' <param name="scard">an instance of our SCard wrapper class.</param>
    '''
    Private Shared Sub PersonalizeConnectedChip(scard As SCard, verbose As Boolean)
        Dim states As Integer() = New Integer(-1) {}
        Dim protocol As UInteger = CUInt(scard_protocol.SCARD_PROTOCOL_UNDEFINED)
        Dim ATRBytes As Byte() = New Byte(-1) {}

        If verbose Then
            Console.WriteLine(Util.GetCurrentTimeString() & " PersonalizeConnectedChip() calling SCardStatus()")
        End If

        Dim scardResult As Long = scard.SCardStatus(states, protocol, ATRBytes)

        If verbose Then
            Console.WriteLine(Util.GetCurrentTimeString() & " SCardStatus() return")
        End If

        Console.WriteLine("SCardStatus result: {0}: {1}", scardResult, Util.Win32ErrorString(CInt(scardResult)))

        If CLng(scard_error.SCARD_S_SUCCESS) <> scardResult Then
            ' unable to get a valid SCardStatus() response; we're done.
            Return
        End If

        Dim cardAbsent As Boolean = False
        Console.Write("SCardStatus() states: ")
        For index As Integer = 0 To states.Length - 1
            Console.Write(states(index))
            Console.Write(" ")
            Console.Write(scard.StringFromState(states(index)))
            Console.Write(" ")

            If scard_state.SCARD_ABSENT = DirectCast(states(index), scard_state) Then
                cardAbsent = True
            End If
        Next
        Console.WriteLine()

        If cardAbsent Then
            ' one of the states is SCARD_ABSENT; we're done.
            Return
        End If

        Console.Write("SCardStatus() ATRBytes (hex): ")
        For index As Integer = 0 To ATRBytes.Length - 1
            Console.Write("{0:X02} ", ATRBytes(index))
        Next
        Console.WriteLine()

        Dim attributesToTry As scard_attr() = New scard_attr() { _
            scard_attr.SCARD_ATTR_VENDOR_NAME, _
            scard_attr.SCARD_ATTR_VENDOR_IFD_SERIAL_NO, _
            scard_attr.SCARD_ATTR_VENDOR_IFD_TYPE, _
            scard_attr.SCARD_ATTR_VENDOR_IFD_VERSION}

        For attribIndex As Integer = 0 To attributesToTry.Length - 1
            Dim scardAttributeBytes As Byte() = New Byte(-1) {}

            If verbose Then
                Console.WriteLine(Util.GetCurrentTimeString() & " PersonalizeConnectedChip() calling SCardGetAttrib()")
            End If

            scardResult = scard.SCardGetAttrib(attributesToTry(attribIndex), scardAttributeBytes)

            If verbose Then
                Console.WriteLine(Util.GetCurrentTimeString() & " SCardGetAttrib() return")
            End If

            If CLng(scard_error.SCARD_S_SUCCESS) <> scardResult Then
                Continue For
            End If

            Dim msg As String = String.Format("SCardGetAttrib({0})", scard.StringFromAttr(CInt(attributesToTry(attribIndex))))
            Console.WriteLine(SCardResultMessage(scardResult, msg))

            Select Case attributesToTry(attribIndex)
                Case scard_attr.SCARD_ATTR_VENDOR_IFD_VERSION
                    ' vendor-supplied interface device version: DWORD in the form 0xMMmmbbbb where
                    '      MM = major version;
                    '      mm = minor version; and
                    '      bbbb = build number:
                    Dim byteCount As Integer = scardAttributeBytes.Length
                    Dim minorVersion As Integer = If(byteCount > 0, scardAttributeBytes(0), 0)
                    Dim majorVersion As Integer = If(byteCount > 1, scardAttributeBytes(1), 0)
                    Dim buildNumber As Integer = 0
                    If byteCount > 3 Then
                        buildNumber = (scardAttributeBytes(2) << 8) + scardAttributeBytes(3)
                    End If

                    Console.WriteLine("  SCARD_ATTR_VENDOR_IFD_VERSION:")
                    Console.WriteLine("    major: " & majorVersion.ToString())
                    Console.WriteLine("    minor: " & minorVersion.ToString())
                    Console.WriteLine("    build: " & buildNumber.ToString())
                    Exit Select
                Case Else
                    ' null-terminate the returned byte for string display.
                    Dim bytesList As List(Of Byte) = New List(Of Byte)(scardAttributeBytes)
                    bytesList.Add(0)
                    scardAttributeBytes = bytesList.ToArray()
                    Console.WriteLine(ASCIIEncoding.UTF8.GetString(scardAttributeBytes))
                    Exit Select
            End Select
        Next

        ' create a bytes for the upcoming SCardTransmit() method.
        ' these particular bytes should function with this type of contact chip:
        '
        '      MPCOS-EMV 16k
        '      GEMPLUS
        '      Datacard part number 806062-002
        '
        Dim sendBytes As Byte() = New Byte() {&H0, &HA4, &H0, &H0}
        Console.Write("sending bytes : ")
        For index As Integer = 0 To sendBytes.Length - 1
            Console.Write("{0:X02} ", sendBytes(index))
        Next
        Console.WriteLine()

        ' for those bytes, we should receive 0x61, 0x12
        Dim receivedBytes As Byte() = New Byte(-1) {}

        If verbose Then
            Console.WriteLine(Util.GetCurrentTimeString() & " PersonalizeConnectedChip() calling SCardTransmit()")
        End If

        scardResult = scard.SCardTransmit(sendBytes, receivedBytes)

        If verbose Then
            Console.WriteLine(Util.GetCurrentTimeString() & " SCardTransmit() return")
        End If

        Console.WriteLine("SCardTransmit result: {0}", scardResult)

        If CLng(scard_error.SCARD_S_SUCCESS) = scardResult AndAlso receivedBytes.Length > 0 Then
            Console.Write("  {0} bytes received: ", receivedBytes.Length)
            For index As Integer = 0 To receivedBytes.Length - 1
                Console.Write("{0:X02} ", receivedBytes(index))
            Next
            Console.WriteLine()
        End If

        ' send the bytes       0x00  0xC0  0x00  0x00  0x12:
        sendBytes = New Byte() {&H0, &HC0, &H0, &H0, &H12}
        Console.Write("sending bytes : ")
        For index As Integer = 0 To sendBytes.Length - 1
            Console.Write("{0:X02} ", sendBytes(index))
        Next
        Console.WriteLine()

        ' for those bytes, we should receive
        '  0x85, 0x10, 0x80, 0x01, 0x3F, 0x00, 0x38, 0x00, 0x00, 0x00,
        '  0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x6B, 0x90, 0x00
        receivedBytes = New Byte(-1) {}

        If verbose Then
            Console.WriteLine(Util.GetCurrentTimeString() & " PersonalizeConnectedChip() calling SCardTransmit()")
        End If

        scardResult = scard.SCardTransmit(sendBytes, receivedBytes)

        If verbose Then
            Console.WriteLine(Util.GetCurrentTimeString() & " PersonalizeConnectedChip() SCardTransmit() return")
        End If

        Console.WriteLine("SCardTransmit result: {0}", scardResult)

        If CLng(scard_error.SCARD_S_SUCCESS) = scardResult AndAlso receivedBytes.Length > 0 Then
            Console.Write("  {0} bytes received: ", receivedBytes.Length)
            For index As Integer = 0 To receivedBytes.Length - 1
                Console.Write("{0:X02} ", receivedBytes(index))
            Next
            Console.WriteLine()
        End If
    End Sub

    ''' <summary>
    ''' Demonstrate the single-wire smartcard calls. The wrapper class we consume
    ''' is similar to the Win32 ::Scard...() functions.
    ''' <returns>
    ''' returns FALSE if either the contact or contactless personalization failed.
    ''' for this sample, we use the result of the SCardConnect() for pass/fail.
    ''' </returns>
    ''' </summary>
    Private Shared Function PersonalizeSmartcard(bidiSpl As BidiSplWrap, verbose As Boolean) As Boolean
        Dim scard As SCard = New SCard(bidiSpl)
        Dim connectedContactOK As Boolean = False

        ' try a 'contact' chip:
        Dim protocol As UInteger = CUInt(scard_protocol.SCARD_PROTOCOL_DEFAULT)

        If verbose Then
            Console.WriteLine(Util.GetCurrentTimeString() & " PersonalizeSmartcard() calling SCardConnect()")
        End If

        Dim scardResult As Long = scard.SCardConnect(scard.ChipConnection.contact, protocol)

        If verbose Then
            Console.WriteLine(Util.GetCurrentTimeString() & " SCardConnect() return")
        End If

        Console.WriteLine("SCardConnect(contact) result: {0}", scardResult)

        If CLng(scard_error.SCARD_S_SUCCESS) = scardResult Then
            connectedContactOK = True
            Dim protocols As String() = scard.StringsFromProtocol(protocol)
            Console.Write("   protocol[s]: ")
            For index As Integer = 0 To protocols.Length - 1
                Console.Write(protocols(index) + " ")
            Next
            Console.WriteLine()

            PersonalizeConnectedChip(scard, verbose)

            If verbose Then
                Console.WriteLine(Util.GetCurrentTimeString() & " PersonalizeSmartcard() calling SCardDisConnect()")
            End If

            scardResult = scard.SCardDisConnect(CInt(scard_disposition.SCARD_LEAVE_CARD))

            If verbose Then
                Console.WriteLine(Util.GetCurrentTimeString() & " SCardDisConnect() return")
            End If

            Console.WriteLine("SCardDisConnect() result: {0}", scardResult)
        End If

        ' try a 'contactless' chip:
        Dim connectedContactlessOK As Boolean = False
        protocol = CUInt(scard_protocol.SCARD_PROTOCOL_DEFAULT)

        If verbose Then
            Console.WriteLine(Util.GetCurrentTimeString() & " PersonalizeSmartcard() calling SCardConnect()")
        End If

        scardResult = scard.SCardConnect(scard.ChipConnection.contactless, protocol)

        If verbose Then
            Console.WriteLine(Util.GetCurrentTimeString() & " SCardConnect() return")
        End If

        Console.WriteLine("SCardConnect(contactless) result: {0}", scardResult)

        If CLng(scard_error.SCARD_S_SUCCESS) = scardResult Then
            connectedContactlessOK = True
            Dim protocols As String() = scard.StringsFromProtocol(protocol)
            Console.Write("   protocol[s]: ")
            For index As Integer = 0 To protocols.Length - 1
                Console.Write(protocols(index) + " ")
            Next
            Console.WriteLine()

            PersonalizeConnectedChip(scard, verbose)

            If verbose Then
                Console.WriteLine(Util.GetCurrentTimeString() & " PersonalizeSmartcard() calling SCardDisConnect()")
            End If

            scardResult = scard.SCardDisConnect(CInt(scard_disposition.SCARD_LEAVE_CARD))

            If verbose Then
                Console.WriteLine(Util.GetCurrentTimeString() & " SCardDisConnect() return")
            End If

            Console.WriteLine("SCardDisConnect() result: {0}", scardResult)
        End If

        Return connectedContactOK AndAlso connectedContactlessOK
    End Function

    Private Shared Sub PollForJobCompletion(bidiSpl As BidiSplWrap, printerJobID As Integer, verbose As Boolean)
        If verbose Then
            Console.WriteLine(Util.GetCurrentTimeString() & " PollForJobCompletion() start")
        End If

        While True

            Dim printerStatusXML As String = bidiSpl.GetPrinterData(strings.PRINTER_MESSAGES)
            Dim printerStatusValues As PrinterStatusValues = Util.ParsePrinterStatusXML(printerStatusXML)
            If 0 <> printerStatusValues._errorCode Then
                Console.WriteLine(printerStatusValues._errorString + " severity: " & printerStatusValues._errorSeverity.ToString())
                Console.WriteLine("canceling printer jobID " & printerStatusValues._printerJobID.ToString())
                CancelJob(bidiSpl, printerStatusValues._printerJobID, printerStatusValues._errorCode, verbose)
                Return
            End If

            Dim xmlFormat As String = strings.JOB_STATUS_XML
            Dim jobInfoXML As String = String.Format(xmlFormat, printerJobID)
            Dim jobStatusXML As String = bidiSpl.GetPrinterData(strings.JOB_STATUS, jobInfoXML)
            Dim jobStatusValues As JobStatusValues = Util.ParseJobStatusXML(jobStatusXML)

            Console.WriteLine(("Printer job id: " & printerJobID & " ") + jobStatusValues._jobState)

            If strings.JOB_ACTIVE <> jobStatusValues._jobState Then
                Return
            End If
            Thread.Sleep(1000)
        End While
    End Sub

    Private Shared Sub StartJob(bidiSpl As BidiSplWrap, ByRef printerJobID As Integer, verbose As Boolean)

        If verbose Then
            Console.WriteLine(Util.GetCurrentTimeString() & " StartJob() start")
        End If

        ' use default input hopper
        ' zero means "do not check print ribbon"
        ' zero means "do not check emboss supplies"
        Dim startJobXml As String = String.Format(strings.STARTJOB_XML, "", 0, 0)

        Dim printerStatusXML As String = bidiSpl.SetPrinterData(strings.STARTJOB, startJobXml)

        Dim printerStatusValues As PrinterStatusValues = Util.ParsePrinterStatusXML(printerStatusXML)
        If 0 <> printerStatusValues._errorCode Then
            Console.WriteLine("StartJob() fail; canceling the job. Printer Error: " + printerStatusValues._errorString)
            CancelJob(bidiSpl, printerStatusValues._printerJobID, printerStatusValues._errorCode, verbose)
            bidiSpl.UnbindDevice()
            Environment.Exit(-1)
        Else
            printerJobID = printerStatusValues._printerJobID
        End If

        If verbose Then
            Console.WriteLine(Util.GetCurrentTimeString() & " StartJob() return")
        End If

    End Sub

    Private Shared Sub SmartcardPark(bidiSpl As BidiSplWrap, verbose As Boolean)
        If verbose Then
            Console.WriteLine(Util.GetCurrentTimeString() & " SmartcardPark() start")
        End If

        Dim printerStatusXML As String = bidiSpl.SetPrinterData(strings.SMARTCARD_PARK)
        Dim printerStatusValues As PrinterStatusValues = Util.ParsePrinterStatusXML(printerStatusXML)
        If 0 <> printerStatusValues._errorCode Then
            Console.WriteLine("SmartcardPark() fail; canceling the job. Printer Error: " + printerStatusValues._errorString)
            CancelJob(bidiSpl, printerStatusValues._printerJobID, printerStatusValues._errorCode, verbose)
            bidiSpl.UnbindDevice()
            Environment.Exit(-1)
        End If

        If verbose Then
            Console.WriteLine(Util.GetCurrentTimeString() & " SmartcardPark() return")
        End If
    End Sub

    Private Shared Sub SmartcardPersonalizationFailed(bidiSpl As BidiSplWrap, printerJobID As Integer, verbose As Boolean)
        Console.WriteLine("SmartcardPersonalizationFailed(); canceling the job.")
        CancelJob(bidiSpl, printerJobID, 0, verbose)
        bidiSpl.UnbindDevice()
        Environment.Exit(-1)
    End Sub

    ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    ' Main()
    ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    Shared Sub Main(args As String())
        Dim commandLineOptions As CommandLineOptions = GetCommandlineOptions(args)

        Dim bidiSpl As New BidiSplWrap()
        bidiSpl.BindDevice(commandLineOptions.printerName)

        If commandLineOptions.verbose Then
            Console.WriteLine(Util.GetCurrentTimeString() & " calling GetPrinterData(version)")
        End If

        Dim sdkVersionXml As String = bidiSpl.GetPrinterData(strings.SDK_VERSION)

        If commandLineOptions.verbose Then
            Console.WriteLine(Util.GetCurrentTimeString() & " GetPrinterData(version) return")
        End If

        Console.WriteLine(Environment.NewLine & "SDK Version: " & Util.ParseSDKVersionXML(sdkVersionXml) & Environment.NewLine)

        If commandLineOptions.verbose Then
            Console.WriteLine(Util.GetCurrentTimeString() & " calling GetPrinterData(options)")
        End If

        ' see if the printer is in the Printer_Ready state:
        Dim printerOptionsXml As String = bidiSpl.GetPrinterData(strings.PRINTER_OPTIONS2)

        If commandLineOptions.verbose Then
            Console.WriteLine(Util.GetCurrentTimeString() & " GetPrinterData(options) return")
        End If

        Dim printerOptionsValues As PrinterOptionsValues = Util.ParsePrinterOptionsXML(printerOptionsXml)
        If "Ready" <> printerOptionsValues._printerStatus Then
            Console.WriteLine(commandLineOptions.printerName & " is not ready.")
            Environment.Exit(-1)
        End If

        ' this application uses the single-wire method for communicating with
        ' a smartcard reader in the printer. If the printer is not configured with
        ' that option, we're done.
        If "Single wire" <> printerOptionsValues._optionSmartcard Then
            Console.WriteLine("printer '" & commandLineOptions.printerName & "' does not have single-wire smartcard option. Exiting.")
            Environment.Exit(-1)
        End If

        ' start an IBidiSpl job:
        Dim printerJobID As Integer = 0
        StartJob(bidiSpl, printerJobID, commandLineOptions.verbose)

        ' Park the card at smartcard station:
        SmartcardPark(bidiSpl, commandLineOptions.verbose)

        ' The card has been parked in the smartcard station. Personalize the smart card.
        Dim personalizedOK As Boolean = False
        Try
            personalizedOK = PersonalizeSmartcard(bidiSpl, commandLineOptions.verbose)
        Catch e As Exception
            Console.WriteLine(e.Message)
        End Try

        If Not personalizedOK Then
            SmartcardPersonalizationFailed(bidiSpl, printerJobID, commandLineOptions.verbose)
        End If

        ' The Smartcard Personalization succeeded; send Resume command
        ResumeJob(bidiSpl, printerJobID, 0, commandLineOptions.verbose)

        If commandLineOptions.print Then
            If commandLineOptions.verbose Then
                Console.WriteLine(Util.GetCurrentTimeString() & " calling PrintTextAndGraphics()")
            End If

            Util.PrintTextAndGraphics(bidiSpl, commandLineOptions.printerName)

            If commandLineOptions.verbose Then
                Console.WriteLine(Util.GetCurrentTimeString() & " PrintTextAndGraphics() return")
            End If
        End If

        ' End the smartcard job
        If commandLineOptions.verbose Then
            Console.WriteLine(Util.GetCurrentTimeString() & " calling endjob")
        End If

        bidiSpl.SetPrinterData(strings.ENDJOB)

        If commandLineOptions.verbose Then
            Console.WriteLine(Util.GetCurrentTimeString() & " return from endjob")
        End If

        If commandLineOptions.pollForJobCompletion Then
            PollForJobCompletion(bidiSpl, printerJobID, commandLineOptions.verbose)
        End If

        If commandLineOptions.verbose Then
            Console.WriteLine(Util.GetCurrentTimeString() & " end.")
        End If

        bidiSpl.UnbindDevice()

    End Sub
End Class
