﻿''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' Copyright (c) Datacard Corporation.  All Rights Reserved.
'
' xps card printer SDK: vb.net status sample
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Imports System.Threading
Imports dxp01sdk

Public Class CommandLineOptions
    Public printerName As String
    Public jobStatus As Boolean
End Class

Class status
    Private Shared Sub Usage()
        Console.WriteLine()
        Console.WriteLine("status.exe demonstrates using interactive mode to retrieve printer options,")
        Console.WriteLine("  printer messages, and job status.")
        Console.WriteLine()
        Console.WriteLine("status.exe -n <printername>")
        Console.WriteLine("  -n <printername>. Required. Try -n ""XPS Card Printer"".")
        Console.WriteLine("  -j Optional. Submit a print job and display job status.")
        Console.WriteLine()
        Environment.Exit(-1)
    End Sub

    Private Shared Function GetCommandlineOptions(args As String()) As CommandLineOptions
        If args.Length = 0 Then
            Usage()
        End If

        Dim commandLineOptions As New CommandLineOptions()
        Dim arguments As New CommandLine.Utility.Arguments(args)

        If String.IsNullOrEmpty(arguments("n")) Then
            Usage()
        End If

        ' we might have a -n with no printer name:
        Dim boolVal As Boolean = False
        If Boolean.TryParse(arguments("n"), boolVal) Then
            Usage()
        End If

        If Not String.IsNullOrEmpty(arguments("j")) Then
            commandLineOptions.jobStatus = True
        End If

        commandLineOptions.printerName = arguments("n")

        Return commandLineOptions
    End Function

    Private Shared Sub CancelJob(bidiSpl As BidiSplWrap, printerJobID As Integer, errorCode As Integer)
        Dim xmlFormat As String = strings.PRINTER_ACTION_XML
        Dim input As String = String.Format(xmlFormat, CInt(Actions.Cancel), printerJobID, errorCode)
        bidiSpl.SetPrinterData(strings.PRINTER_ACTION, input)
    End Sub

    Private Shared Sub PollForJobCompletion(bidiSpl As BidiSplWrap, printerJobID As Integer)
        Console.WriteLine("JobStatus:")
        While True
            Dim temp As String = strings.JOB_STATUS_XML
            Dim jobInfoXML As String = String.Format(temp, printerJobID)
            Dim jobStatusXML As String = bidiSpl.GetPrinterData(strings.JOB_STATUS, jobInfoXML)
            Dim jobStatusValues As JobStatusValues = Util.ParseJobStatusXML(jobStatusXML)
            Console.WriteLine("  ClientID:          " + jobStatusValues._clientID)
            Console.WriteLine("  WindowsJobID:      " + jobStatusValues._windowsJobID.ToString())
            Console.WriteLine("  PrinterJobID:      " + jobStatusValues._printerJobID.ToString())
            Console.WriteLine("  JobState:          " + jobStatusValues._jobState)
            Console.WriteLine("  JobRestartCount:   " + jobStatusValues._jobRestartCount.ToString())
            Console.WriteLine()
            If strings.JOB_ACTIVE <> jobStatusValues._jobState Then
                Console.WriteLine()
                Return
            End If
            Thread.Sleep(1000)
        End While
    End Sub

    Private Shared Sub StartJob(bidiSpl As BidiSplWrap, ByRef printerJobID As Integer)
        Dim printerStatusXML As String = bidiSpl.SetPrinterData(strings.STARTJOB)
        Dim printerStatusValues As PrinterStatusValues = Util.ParsePrinterStatusXML(printerStatusXML)
        If 0 <> printerStatusValues._errorCode Then
            ' the StartJob failed; send a Cancel(); do NOT send an EndJob():
            CancelJob(bidiSpl, printerStatusValues._printerJobID, printerStatusValues._errorCode)
            Console.WriteLine("Canceling the job. Printer Error: " + printerStatusValues._errorString)
            bidiSpl.UnbindDevice()
            Environment.Exit(-1)
        Else
            printerJobID = printerStatusValues._printerJobID
        End If
    End Sub

    Private Shared Sub DisplayPrinterOptionsValues(vals As PrinterOptionsValues)
        Console.WriteLine("Options:")
        Console.WriteLine("  ColorPrintResolution:      " & vals._colorPrintResolution)
        Console.WriteLine("  ConnectionPortType:        " & vals._connectionPortType)
        Console.WriteLine("  ConnectionProtocol:        " & vals._connectionProtocol)
        Console.WriteLine("  EmbosserVersion:           " & vals._embosserVersion)
        Console.WriteLine("  LockState:                 " & vals._lockState)
        Console.WriteLine("  ModuleEmbosser:            " & vals._moduleEmbosser)
        Console.WriteLine("  MonochromePrintResolution: " & vals._monochromePrintResolution)
        Console.WriteLine("  OptionDuplex:              " & vals._optionDuplex)
        Console.WriteLine("  OptionInputhopper:         " & vals._optionInputhopper)
        Console.WriteLine("  OptionLocks:               " & vals._optionLocks)
        Console.WriteLine("  OptionMagstripe:           " & vals._optionMagstripe)
        Console.WriteLine("  OptionSmartcard:           " & vals._optionSmartcard)
        Console.WriteLine("  PrinterAddress:            " & vals._printerAddress)
        Console.WriteLine("  PrinterMessageNumber:      " & Convert.ToString(vals._printerMessageNumber))
        Console.WriteLine("  PrinterModel:              " & vals._printerModel)
        Console.WriteLine("  PrinterSerialNumber:       " & vals._printerSerialNumber)
        Console.WriteLine("  PrinterStatus:             " & vals._printerStatus)
        Console.WriteLine("  PrinterVersion:            " & vals._printerVersion)
        Console.WriteLine("  PrintHead:                 " & vals._printHead)
        Console.WriteLine("  TopcoatPrintResolution:    " & vals._topcoatPrintResolution)
        Console.WriteLine()
    End Sub

    Private Shared Sub DisplayPrinterStatusValues(vals As PrinterStatusValues)
        Console.WriteLine("Status:")
        Console.WriteLine("  ClientID:      " & Convert.ToString(vals._clientID))
        Console.WriteLine("  ErrorCode:     " & Convert.ToString(vals._errorCode))
        Console.WriteLine("  ErrorSeverity: " & Convert.ToString(vals._errorSeverity))
        Console.WriteLine("  ErrorString:   " & Convert.ToString(vals._errorString))
        Console.WriteLine("  PrinterData:   " & Convert.ToString(vals._dataFromPrinter))
        Console.WriteLine("  PrinterJobID:  " & Convert.ToString(vals._printerJobID))
        Console.WriteLine("  WindowsJobID:  " & Convert.ToString(vals._windowsJobID))
        Console.WriteLine()
    End Sub

    Private Shared Sub DisplayPrinterCounterValues(vals As PrinterCounterStatus)
        Console.WriteLine("Counts:")
        Console.WriteLine("  CardsPickedSinceCleaningCard:  " & Convert.ToString(vals._cardsPickedSinceCleaningCard))
        Console.WriteLine("  CleaningCardsRun:              " & Convert.ToString(vals._cleaningCardsRun))
        Console.WriteLine("  CurrentCompleted:              " & Convert.ToString(vals._currentCompleted))
        Console.WriteLine("  CurrentLost:                   " & Convert.ToString(vals._currentLost))
        Console.WriteLine("  CurrentPicked:                 " & Convert.ToString(vals._currentPicked))
        Console.WriteLine("  CurrentPickedExceptionSlot:    " & Convert.ToString(vals._currentPickedExceptionSlot))
        Console.WriteLine("  CurrentPickedInputHopper1:     " & Convert.ToString(vals._currentPickedInputHopper1))
        Console.WriteLine("  CurrentPickedInputHopper2:     " & Convert.ToString(vals._currentPickedInputHopper2))
        Console.WriteLine("  CurrentPickedInputHopper3:     " & Convert.ToString(vals._currentPickedInputHopper3))
        Console.WriteLine("  CurrentPickedInputHopper4:     " & Convert.ToString(vals._currentPickedInputHopper4))
        Console.WriteLine("  CurrentPickedInputHopper5:     " & Convert.ToString(vals._currentPickedInputHopper5))
        Console.WriteLine("  CurrentPickedInputHopper6:     " & Convert.ToString(vals._currentPickedInputHopper6))
        Console.WriteLine("  CurrentRejected:               " & Convert.ToString(vals._currentRejected))
        Console.WriteLine("  PrinterStatus:                 " & vals._printerStatus)
        Console.WriteLine("  TotalCompleted:                " & Convert.ToString(vals._totalCompleted))
        Console.WriteLine("  TotalLost:                     " & Convert.ToString(vals._totalLost))
        Console.WriteLine("  TotalPicked:                   " & Convert.ToString(vals._totalPicked))
        Console.WriteLine("  TotalPickedExceptionSlot:      " & Convert.ToString(vals._totalPickedExceptionSlot))
        Console.WriteLine("  TotalPickedInputHopper1:       " & Convert.ToString(vals._totalPickedInputHopper1))
        Console.WriteLine("  TotalPickedInputHopper2:       " & Convert.ToString(vals._totalPickedInputHopper2))
        Console.WriteLine("  TotalPickedInputHopper3:       " & Convert.ToString(vals._totalPickedInputHopper3))
        Console.WriteLine("  TotalPickedInputHopper4:       " & Convert.ToString(vals._totalPickedInputHopper4))
        Console.WriteLine("  TotalPickedInputHopper5:       " & Convert.ToString(vals._totalPickedInputHopper5))
        Console.WriteLine("  TotalPickedInputHopper6:       " & Convert.ToString(vals._totalPickedInputHopper6))
        Console.WriteLine("  TotalRejected:                 " & Convert.ToString(vals._totalRejected))
        Console.WriteLine()
    End Sub

    Private Shared Sub DisplaySuppliesValues(vals As SuppliesValues)
        Console.WriteLine("Supplies:")

        Console.WriteLine("  IndentRibbon:          " & Convert.ToString(vals._indentRibbon))
        Console.WriteLine("  IndentRibbonRemaining: " & Convert.ToString(vals._indentRibbonRemaining) & "%")
        Console.WriteLine("  PrinterStatus:         " & vals._printerStatus)
        Console.WriteLine("  PrintRibbonType:       " & Convert.ToString(vals._printRibbonType))
        Console.WriteLine("  RibbonLotCode:         " & vals._printRibbonLotCode)
        Console.WriteLine("  RibbonPartNumber:      " & Convert.ToString(vals._printRibbonPartNumber))
        Console.WriteLine("  RibbonRemaining:       " & Convert.ToString(vals._ribbonRemaining) & "%")
        Console.WriteLine("  RibbonSerialNumber:    " & vals._printRibbonSerialNumber)
        Console.WriteLine("  TopperRibbonRemaining: " & Convert.ToString(vals._topperRibbonRemaining) & "%")
        Console.WriteLine("  TopperRibbonType:      " & Convert.ToString(vals._topperRibbonType))

        Console.WriteLine()
    End Sub

    Private Shared Sub CardJobWithStatus(bidiSpl As BidiSplWrap, commandLineOptions As CommandLineOptions, printerOptionsValues As PrinterOptionsValues)
        If "Ready" <> printerOptionsValues._printerStatus Then
            Console.WriteLine(commandLineOptions.printerName & " is not ready.")
            bidiSpl.UnbindDevice()
            Environment.Exit(-1)
        End If

        Dim printerJobID As Integer = 0
        StartJob(bidiSpl, printerJobID)

        Dim printerStatusXML As String = bidiSpl.GetPrinterData(strings.PRINTER_MESSAGES)
        Dim printerStatusValues As PrinterStatusValues = Util.ParsePrinterStatusXML(printerStatusXML)
        DisplayPrinterStatusValues(printerStatusValues)

        bidiSpl.SetPrinterData(strings.ENDJOB)

        PollForJobCompletion(bidiSpl, printerJobID)
    End Sub

    ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    ' Main()
    ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    Public Shared Sub Main(args As String())
        Dim commandLineOptions As CommandLineOptions = GetCommandlineOptions(args)

        Dim bidiSpl As New BidiSplWrap()
        bidiSpl.BindDevice(commandLineOptions.printerName)

        Dim sdkVersion As String = bidiSpl.GetPrinterData(strings.SDK_VERSION)
        Console.WriteLine(Environment.NewLine & "SDK Version: " & Util.ParseSDKVersionXML(sdkVersion) & Environment.NewLine)

        Dim printerStatusXML As String = bidiSpl.GetPrinterData(strings.PRINTER_MESSAGES)
        Dim printerStatusValues As PrinterStatusValues = Util.ParsePrinterStatusXML(printerStatusXML)
        DisplayPrinterStatusValues(printerStatusValues)

        Dim printerOptionsXML As String = bidiSpl.GetPrinterData(strings.PRINTER_OPTIONS2)
        Dim printerOptionsValues As PrinterOptionsValues = Util.ParsePrinterOptionsXML(printerOptionsXML)
        DisplayPrinterOptionsValues(printerOptionsValues)

        Dim printerCardCountXML As String = bidiSpl.GetPrinterData(strings.COUNTER_STATUS2)
        Dim printerCounterStatusValues As PrinterCounterStatus = Util.ParsePrinterCounterStatusXML(printerCardCountXML)
        DisplayPrinterCounterValues(printerCounterStatusValues)

        Dim suppliesXML As String = bidiSpl.GetPrinterData(strings.SUPPLIES_STATUS2)
        Dim suppliesValues As SuppliesValues = Util.ParseSuppliesXML(suppliesXML)
        DisplaySuppliesValues(suppliesValues)

        If commandLineOptions.jobStatus Then
            CardJobWithStatus(bidiSpl, commandLineOptions, printerOptionsValues)
        End If

        bidiSpl.UnbindDevice()
    End Sub
End Class

